# -*- coding: utf-8 -*-
"""CLI related functions and classes for testing."""

from __future__ import unicode_literals

import argparse
import io
import operator
import os

from plaso.cli import tools

from tests import test_lib as shared_test_lib


class SortedArgumentsHelpFormatter(argparse.HelpFormatter):
  """Class that implements an argparse help formatter with sorted arguments."""

  def add_arguments(self, actions):
    """Adds arguments.

    Args:
      actions (list[argparse._StoreAction]): command line actions.
    """
    actions = sorted(actions, key=operator.attrgetter('option_strings'))
    super(SortedArgumentsHelpFormatter, self).add_arguments(actions)


class TestOptions(object):
  """Class to define test options."""


class TestOutputWriter(tools.FileObjectOutputWriter):
  """Test output writer that reads and writes strings."""

  def __init__(self, encoding='utf-8'):
    """Initializes the output writer object.

    Args:
      encoding (Optional[str]): output encoding.
    """
    file_object = io.StringIO()
    super(TestOutputWriter, self).__init__(file_object, encoding=encoding)
    self._read_offset = 0

  def Write(self, string):
    """Writes a string to the output.

    Args:
      string (str): output.
    """
    # To mimic the behavior of the parent FileObjectOutputWriter, we encode the
    # provided string into the specified encoding, replacing invalid characters.
    byte_stream = string.encode(self._encoding, errors='replace')
    string = byte_stream.decode(self._encoding, errors='strict')

    self._file_object.write(string)

  def ReadOutput(self):
    """Reads all output added since the last call to ReadOutput.

    Returns:
      str: output data.
    """
    self._file_object.seek(self._read_offset, os.SEEK_SET)
    output_data = self._file_object.read()
    self._read_offset = self._file_object.tell()

    return output_data


class TestBinaryOutputWriter(tools.FileObjectOutputWriter):
  """Test output writer that reads and writes bytes."""

  def __init__(self, encoding='utf-8'):
    """Initializes the output writer object.

    Args:
      encoding (Optional[str]): output encoding.
    """
    file_object = io.BytesIO()
    super(TestBinaryOutputWriter, self).__init__(file_object, encoding=encoding)
    self._read_offset = 0

  def ReadOutput(self):
    """Reads all output added since the last call to ReadOutput.

    Returns:
      bytes: output data.
    """
    self._file_object.seek(self._read_offset, os.SEEK_SET)
    output_data = self._file_object.read()
    self._read_offset = self._file_object.tell()

    return output_data


class CLIToolTestCase(shared_test_lib.BaseTestCase):
  """The unit test case for a CLI tool."""

  def _RunArgparseFormatHelp(self, argument_parser):
    """Runs argparse.format_help() with test conditions.

    Args:
      argument_parser (argparse.ArgumentParser): argument parser.

    Returns:
      bytes: output of argparse.format_help().
    """
    columns_environment_variable = os.environ.get('COLUMNS', None)
    os.environ['COLUMNS'] = '80'

    try:
      output = argument_parser.format_help()
    finally:
      if columns_environment_variable:
        os.environ['COLUMNS'] = columns_environment_variable
      else:
        del os.environ['COLUMNS']

    return output



# -*- coding: utf-8 -*-
"""Shared functionality for Elasticsearch output modules."""

from __future__ import unicode_literals

import os
import logging

from dfvfs.serializer.json_serializer import JsonPathSpecSerializer

try:
  import elasticsearch
except ImportError:
  elasticsearch = None

from plaso.lib import errors
from plaso.lib import py2to3
from plaso.lib import timelib
from plaso.output import interface
from plaso.output import logger

# Configure the Elasticsearch logger.
if elasticsearch:
  elastic_logger = logging.getLogger('elasticsearch.trace')
  elastic_logger.setLevel(logging.WARNING)


class SharedElasticsearchOutputModule(interface.OutputModule):
  """Shared functionality for an Elasticsearch output module."""

  # pylint: disable=abstract-method

  NAME = 'elastic_shared'

  _DEFAULT_DOCUMENT_TYPE = 'plaso_event'

  _DEFAULT_FLUSH_INTERVAL = 1000

  # Number of seconds to wait before a request to Elasticsearch is timed out.
  _DEFAULT_REQUEST_TIMEOUT = 300

  def __init__(self, output_mediator):
    """Initializes an Elasticsearch output module.

    Args:
      output_mediator (OutputMediator): mediates interactions between output
          modules and other components, such as storage and dfvfs.
    """
    super(SharedElasticsearchOutputModule, self).__init__(output_mediator)
    self._client = None
    self._document_type = self._DEFAULT_DOCUMENT_TYPE
    self._event_documents = []
    self._flush_interval = self._DEFAULT_FLUSH_INTERVAL
    self._host = None
    self._index_name = None
    self._number_of_buffered_events = 0
    self._password = None
    self._port = None
    self._username = None
    self._use_ssl = None
    self._ca_certs = None
    self._url_prefix = None

  def _Connect(self):
    """Connects to an Elasticsearch server."""
    elastic_host = {'host': self._host, 'port': self._port}

    if self._url_prefix:
      elastic_host['url_prefix'] = self._url_prefix

    elastic_http_auth = None
    if self._username is not None:
      elastic_http_auth = (self._username, self._password)

    self._client = elasticsearch.Elasticsearch(
        [elastic_host],
        http_auth=elastic_http_auth,
        use_ssl=self._use_ssl,
        ca_certs=self._ca_certs
    )

    logger.debug(
        ('Connected to Elasticsearch server: {0:s} port: {1:d}'
         'URL prefix {2!s}.').format(self._host, self._port, self._url_prefix))

  def _CreateIndexIfNotExists(self, index_name, mappings):
    """Creates an Elasticsearch index if it does not exist.

    Args:
      index_name (str): mame of the index.
      mappings (dict[str, object]): mappings of the index.

    Raises:
      RuntimeError: if the Elasticsearch index cannot be created.
    """
    try:
      if not self._client.indices.exists(index_name):
        self._client.indices.create(
            body={'mappings': mappings}, index=index_name)

    except elasticsearch.exceptions.ConnectionError as exception:
      raise RuntimeError(
          'Unable to create Elasticsearch index with error: {0!s}'.format(
              exception))

  def _FlushEvents(self):
    """Inserts the buffered event documents into Elasticsearch."""
    try:
      # pylint: disable=unexpected-keyword-arg
      # pylint does not recognizes request_timeout as a valid kwarg. According
      # to http://elasticsearch-py.readthedocs.io/en/master/api.html#timeout
      # it should be supported.
      self._client.bulk(
          body=self._event_documents, doc_type=self._document_type,
          index=self._index_name, request_timeout=self._DEFAULT_REQUEST_TIMEOUT)

    except (
        ValueError,
        elasticsearch.exceptions.ElasticsearchException) as exception:
      # Ignore problematic events
      logger.warning('Unable to bulk insert with error: {0!s}'.format(
          exception))

    logger.debug('Inserted {0:d} events into Elasticsearch'.format(
        self._number_of_buffered_events))

    self._event_documents = []
    self._number_of_buffered_events = 0

  def _GetSanitizedEventValues(self, event, event_data, event_tag):
    """Sanitizes the event for use in Elasticsearch.

    The event values need to be sanitized to prevent certain values from
    causing problems when indexing with Elasticsearch. For example the path
    specification is a nested dictionary which will cause problems for
    Elasticsearch automatic indexing.

    Args:
      event (EventObject): event.
      event_data (EventData): event data.
      event_tag (EventTag): event tag.

    Returns:
      dict[str, object]: sanitized event values.

    Raises:
      NoFormatterFound: if no event formatter can be found to match the data
          type in the event data.
    """
    event_values = {}
    for attribute_name, attribute_value in event_data.GetAttributes():
      # TODO: remove regvalue, which is kept for backwards compatibility.
      # Ignore the regvalue attribute as it cause issues when indexing.
      if attribute_name == 'regvalue':
        continue

      if attribute_name == 'pathspec':
        try:
          attribute_value = JsonPathSpecSerializer.WriteSerialized(
              attribute_value)
        except TypeError:
          continue
      event_values[attribute_name] = attribute_value

      if isinstance(attribute_value, py2to3.BYTES_TYPE):
        # Some parsers have written bytes values to storage.
        attribute_value = attribute_value.decode('utf-8', 'replace')
        logger.warning(
            'Found bytes value for attribute "{0:s}" for data type: '
            '{1!s}. Value was converted to UTF-8: "{2:s}"'.format(
                attribute_name, event_data.data_type, attribute_value))
        event_values[attribute_name] = attribute_value

    # Add a string representation of the timestamp.
    try:
      attribute_value = timelib.Timestamp.RoundToSeconds(event.timestamp)
    except TypeError as exception:
      logger.warning((
          'Unable to round timestamp {0!s}. error: {1!s}. '
          'Defaulting to 0').format(event.timestamp, exception))
      attribute_value = 0

    attribute_value = timelib.Timestamp.CopyToIsoFormat(
        attribute_value, timezone=self._output_mediator.timezone)
    event_values['datetime'] = attribute_value

    event_values['timestamp'] = event.timestamp
    event_values['timestamp_desc'] = event.timestamp_desc

    message, _ = self._output_mediator.GetFormattedMessages(event_data)
    if message is None:
      data_type = getattr(event_data, 'data_type', 'UNKNOWN')
      raise errors.NoFormatterFound(
          'Unable to find event formatter for: {0:s}.'.format(data_type))

    event_values['message'] = message

    # Tags needs to be a list for Elasticsearch to index correctly.
    labels = []
    if event_tag:
      try:
        labels = list(event_tag.labels)
      except (AttributeError, KeyError):
        pass

    event_values['tag'] = labels

    source_short, source = self._output_mediator.GetFormattedSources(
        event, event_data)
    if source is None or source_short is None:
      data_type = getattr(event_data, 'data_type', 'UNKNOWN')
      raise errors.NoFormatterFound(
          'Unable to find event formatter for: {0:s}.'.format(data_type))

    event_values['source_short'] = source_short
    event_values['source_long'] = source

    return event_values

  def _InsertEvent(self, event, event_data, event_tag):
    """Inserts an event.

    Events are buffered in the form of documents and inserted to Elasticsearch
    when the flush interval (threshold) has been reached.

    Args:
      event (EventObject): event.
      event_data (EventData): event data.
      event_tag (EventTag): event tag.
    """
    event_document = {'index': {
        '_index': self._index_name, '_type': self._document_type}}
    event_values = self._GetSanitizedEventValues(event, event_data, event_tag)

    self._event_documents.append(event_document)
    self._event_documents.append(event_values)
    self._number_of_buffered_events += 1

    if self._number_of_buffered_events > self._flush_interval:
      self._FlushEvents()

  def Close(self):
    """Closes connection to Elasticsearch.

    Inserts any remaining buffered event documents.
    """
    self._FlushEvents()

    self._client = None

  def SetDocumentType(self, document_type):
    """Sets the document type.

    Args:
      document_type (str): document type.
    """
    self._document_type = document_type
    logger.debug('Elasticsearch document type: {0:s}'.format(document_type))

  def SetFlushInterval(self, flush_interval):
    """Set the flush interval.

    Args:
      flush_interval (int): number of events to buffer before doing a bulk
          insert.
    """
    self._flush_interval = flush_interval
    logger.debug('Elasticsearch flush interval: {0:d}'.format(flush_interval))

  def SetIndexName(self, index_name):
    """Set the index name.

    Args:
      index_name (str): name of the index.
    """
    self._index_name = index_name
    logger.debug('Elasticsearch index name: {0:s}'.format(index_name))

  def SetPassword(self, password):
    """Set the password.

    Args:
      password (str): password to authenticate with.
    """
    self._password = password
    logger.debug('Elastic password: ********')

  def SetServerInformation(self, server, port):
    """Set the server information.

    Args:
      server (str): IP address or hostname of the server.
      port (int): Port number of the server.
    """
    self._host = server
    self._port = port
    logger.debug('Elasticsearch server: {0!s} port: {1:d}'.format(
        server, port))

  def SetUsername(self, username):
    """Sets the username.

    Args:
      username (str): username to authenticate with.
    """
    self._username = username
    logger.debug('Elasticsearch username: {0!s}'.format(username))

  def SetUseSSL(self, use_ssl):
    """Sets the use of ssl.

    Args:
      use_ssl (bool): enforces use of ssl.
    """
    self._use_ssl = use_ssl
    logger.debug('Elasticsearch use_ssl: {0!s}'.format(use_ssl))

  def SetCACertificatesPath(self, ca_certificates_path):
    """Sets the path to the CA certificates.

    Args:
      ca_certificates_path (str): path to file containing a list of root
        certificates to trust.

    Raises:
      BadConfigOption: if the CA certificates file does not exist.
    """
    if not ca_certificates_path:
      return

    if not os.path.exists(ca_certificates_path):
      raise errors.BadConfigOption(
          'No such certificate file: {0:s}.'.format(ca_certificates_path))

    self._ca_certs = ca_certificates_path
    logger.debug('Elasticsearch ca_certs: {0!s}'.format(ca_certificates_path))

  def SetURLPrefix(self, url_prefix):
    """Sets the URL prefix.

    Args:
      url_prefix (str): URL prefix.
    """
    self._url_prefix = url_prefix
    logger.debug('Elasticsearch URL prefix: {0!s}')

  def WriteEventBody(self, event, event_data, event_tag):
    """Writes event values to the output.

    Args:
      event (EventObject): event.
      event_data (EventData): event data.
      event_tag (EventTag): event tag.
    """
    self._InsertEvent(event, event_data, event_tag)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""The image export command line tool."""

from __future__ import unicode_literals

import logging
import multiprocessing
import os
import sys

from plaso import dependencies
from plaso.cli import image_export_tool
from plaso.lib import errors


def Main():
  """The main function.

  Returns:
    bool: True if successful or False otherwise.
  """
  tool = image_export_tool.ImageExportTool()

  if not tool.ParseArguments(sys.argv[1:]):
    return False

  if tool.show_troubleshooting:
    print('Using Python version {0!s}'.format(sys.version))
    print()
    print('Path: {0:s}'.format(os.path.abspath(__file__)))
    print()
    print(tool.GetVersionInformation())
    print()
    dependencies.CheckDependencies(verbose_output=True)

    print('Also see: https://plaso.readthedocs.io/en/latest/sources/user/'
          'Troubleshooting.html')
    return True

  if tool.list_signature_identifiers:
    tool.ListSignatureIdentifiers()
    return True

  if not tool.has_filters:
    logging.warning('No filter defined exporting all files.')

  # TODO: print more status information like PrintOptions.
  tool.PrintFilterCollection()

  try:
    tool.ProcessSources()

  except (KeyboardInterrupt, errors.UserAbort):
    logging.warning('Aborted by user.')
    return False

  except errors.BadConfigOption as exception:
    logging.warning(exception)
    return False

  except errors.SourceScannerError as exception:
    logging.warning((
        'Unable to scan for a supported filesystem with error: {0!s}\n'
        'Most likely the image format is not supported by the '
        'tool.').format(exception))
    return False

  return True


if __name__ == '__main__':
  # For PyInstaller sake we need to define this directly after "__main__".
  # https://github.com/pyinstaller/pyinstaller/wiki/Recipe-Multiprocessing
  multiprocessing.freeze_support()

  if not Main():
    sys.exit(1)
  else:
    sys.exit(0)



# -*- coding: utf-8 -*-
"""The Sophos Anti-Virus log (SAV.txt) file event formatter."""

from __future__ import unicode_literals

from plaso.formatters import interface
from plaso.formatters import manager


class SophosAVLogFormatter(interface.ConditionalEventFormatter):
  """Formatter for a Sophos Anti-Virus log (SAV.txt) event data."""

  DATA_TYPE = 'sophos:av:log'

  FORMAT_STRING_PIECES = [
      '{text}']

  SOURCE_LONG = 'Sophos Anti-Virus log'
  SOURCE_SHORT = 'LOG'


manager.FormattersManager.RegisterFormatter(SophosAVLogFormatter)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the MD5 hasher."""

from __future__ import unicode_literals

import unittest

from plaso.analyzers.hashers import md5

from tests.analyzers.hashers import test_lib


class MD5Test(test_lib.HasherTestCase):
  """Tests the MD5 hasher."""

  def testFileHashMatchesEmptyFile(self):
    """Tests that hasher matches the hash of an empty file."""
    hasher = md5.MD5Hasher()
    self._AssertTestPathStringDigestMatch(
        hasher, ['empty_file'], 'd41d8cd98f00b204e9800998ecf8427e')

  def testFileHashMatchesKnownFile(self):
    """Tests that hasher matches the hash of a known file."""
    hasher = md5.MD5Hasher()
    self._AssertTestPathStringDigestMatch(
        hasher, ['ímynd.dd'], 'd73c51f10c7ee6a681b7b619ccc6f1c4')


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the SAM Users Account information plugin."""

from __future__ import unicode_literals

import unittest

from plaso.formatters import winreg  # pylint: disable=unused-import
from plaso.lib import definitions
from plaso.parsers.winreg_plugins import sam_users

from tests.parsers.winreg_plugins import test_lib


class SAMUsersWindowsRegistryPluginTest(test_lib.RegistryPluginTestCase):
  """Tests the SAM Users Account information plugin."""

  def testFilters(self):
    """Tests the FILTERS class attribute."""
    plugin = sam_users.SAMUsersWindowsRegistryPlugin()

    key_path = 'HKEY_LOCAL_MACHINE\\SAM\\SAM\\Domains\\Account\\Users'
    self._AssertFiltersOnKeyPath(plugin, key_path)

    self._AssertNotFiltersOnKeyPath(plugin, 'HKEY_LOCAL_MACHINE\\Bogus')

  def testProcess(self):
    """Tests the Process function."""
    test_file_entry = self._GetTestFileEntry(['SAM'])
    key_path = 'HKEY_LOCAL_MACHINE\\SAM\\SAM\\Domains\\Account\\Users'

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin = sam_users.SAMUsersWindowsRegistryPlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    self.assertEqual(storage_writer.number_of_warnings, 0)
    self.assertEqual(storage_writer.number_of_events, 7)

    events = list(storage_writer.GetEvents())

    event = events[0]

    self.CheckTimestamp(event.timestamp, '2014-09-24 03:36:06.358837')
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_WRITTEN)

    event_data = self._GetEventDataOfEvent(storage_writer, event)

    self.assertEqual(event_data.data_type, 'windows:registry:sam_users')
    self.assertEqual(event_data.account_rid, 500)
    self.assertEqual(event_data.login_count, 6)
    self.assertEqual(event_data.username, 'Administrator')

    expected_message = (
        '[{0:s}] '
        'Username: Administrator '
        'Comments: Built-in account for administering the computer/domain '
        'RID: 500 '
        'Login count: 6').format(key_path)
    expected_short_message = (
        'Administrator '
        'RID: 500 '
        'Login count: 6')

    self._TestGetMessageStrings(
        event_data, expected_message, expected_short_message)

    # Test SAMUsersWindowsRegistryEvent.
    event = events[1]

    self.CheckTimestamp(event.timestamp, '2010-11-20 21:48:12.569244')
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_LAST_LOGIN)

    event_data = self._GetEventDataOfEvent(storage_writer, event)

    self.assertEqual(event_data.data_type, 'windows:registry:sam_users')

    expected_message = (
        '[{0:s}] '
        'Username: Administrator '
        'Comments: Built-in account for administering the computer/domain '
        'RID: 500 '
        'Login count: 6').format(key_path)
    expected_short_message = (
        'Administrator '
        'RID: 500 '
        'Login count: 6')

    self._TestGetMessageStrings(
        event_data, expected_message, expected_short_message)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the tagging analysis plugin."""

from __future__ import unicode_literals

import unittest

from plaso.analysis import tagging
from plaso.lib import definitions
from plaso.lib import timelib

from tests.analysis import test_lib


class TaggingAnalysisPluginTest(test_lib.AnalysisPluginTestCase):
  """Tests the tagging analysis plugin."""

  # pylint: disable=protected-access

  _TEST_EVENTS = [
      {'data_type': 'windows:prefetch',
       'timestamp': timelib.Timestamp.CopyFromString('2015-05-01 15:12:00'),
       'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN},
      {'data_type': 'chrome:history:file_downloaded',
       'timestamp': timelib.Timestamp.CopyFromString('2015-05-01 05:06:00'),
       'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN},
      {'data_type': 'something_else',
       'timestamp': timelib.Timestamp.CopyFromString('2015-02-19 08:00:01'),
       'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN},
      {'data_type': 'windows:evt:record',
       'event_identifier': 538,
       'source_name': 'Security',
       'timestamp': timelib.Timestamp.CopyFromString('2016-05-25 13:00:06'),
       'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN},
      {'body': 'this is a message',
       'data_type': 'windows:evt:record',
       'event_identifier': 16,
       'timestamp': timelib.Timestamp.CopyFromString('2016-05-25 13:00:06'),
       'source_name': 'Messaging',
       'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN}]

  def testExamineEventAndCompileReport(self):
    """Tests the ExamineEvent and CompileReport functions."""
    test_file_path = self._GetTestFilePath(['tagging_file', 'valid.txt'])
    self._SkipIfPathNotExists(test_file_path)

    plugin = tagging.TaggingAnalysisPlugin()
    plugin.SetAndLoadTagFile(test_file_path)

    storage_writer = self._AnalyzeEvents(self._TEST_EVENTS, plugin)

    self.assertEqual(len(storage_writer.analysis_reports), 1)
    self.assertEqual(storage_writer.number_of_event_tags, 4)

    report = storage_writer.analysis_reports[0]
    self.assertIsNotNone(report)

    expected_text = 'Tagging plugin produced 4 tags.\n'
    self.assertEqual(report.text, expected_text)

    labels = []
    for event_tag in storage_writer.GetEventTags():
      labels.extend(event_tag.labels)

    self.assertEqual(len(labels), 5)

    # This is from a tag rule declared in objectfilter syntax.
    self.assertIn('application_execution', labels)
    # This is from a tag rule declared in dotty syntax.
    self.assertIn('login_attempt', labels)
    # This is from a rule using the "contains" operator
    self.assertIn('text_contains', labels)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the parsers manager."""

from __future__ import unicode_literals

import unittest

from plaso.parsers import interface
from plaso.parsers import manager
from plaso.parsers import plugins

from tests import test_lib as shared_test_lib


class TestParser(interface.BaseParser):
  """Test parser."""

  NAME = 'test_parser'
  DESCRIPTION = 'Test parser.'

  # pylint: disable=unused-argument
  def Parse(self, parser_mediator, **kwargs):
    """Parses the file entry and extracts events.

    Args:
      parser_mediator (ParserMediator): parser mediator.
    """
    return


class TestParserWithPlugins(interface.BaseParser):
  """Test parser with plugins."""

  NAME = 'test_parser_with_plugins'
  DESCRIPTION = 'Test parser with plugins.'

  _plugin_classes = {}

  # pylint: disable=unused-argument
  def Parse(self, parser_mediator, **kwargs):
    """Parses the file entry and extracts events.

    Args:
      parser_mediator (ParserMediator): parser mediator.
    """
    return


class TestPlugin(plugins.BasePlugin):
  """Test plugin."""

  NAME = 'test_plugin'
  DESCRIPTION = 'Test plugin.'

  # pylint: disable=unused-argument
  def Process(self, parser_mediator, **kwargs):
    """Evaluates if this is the correct plugin and processes data accordingly.

    Args:
      parser_mediator (ParserMediator): parser mediator.
      kwargs (dict[str, object]): Depending on the plugin they may require
          different sets of arguments to be able to evaluate whether or not
          this is the correct plugin.

    Raises:
      ValueError: When there are unused keyword arguments.
    """
    return


class ParsersManagerTest(shared_test_lib.BaseTestCase):
  """Tests for the parsers manager."""

  # pylint: disable=protected-access

  # TODO: add tests for CreateSignatureScanner.

  def testParserRegistration(self):
    """Tests the RegisterParser and DeregisterParser functions."""
    number_of_parsers = len(manager.ParsersManager._parser_classes)

    manager.ParsersManager.RegisterParser(TestParser)
    self.assertEqual(
        len(manager.ParsersManager._parser_classes),
        number_of_parsers + 1)

    with self.assertRaises(KeyError):
      manager.ParsersManager.RegisterParser(TestParser)

    manager.ParsersManager.DeregisterParser(TestParser)
    self.assertEqual(
        len(manager.ParsersManager._parser_classes),
        number_of_parsers)

  # TODO: add tests for GetFormatsWithSignatures.

  def testCheckParserNames(self):
    """Tests the CheckFilterExpression function."""
    TestParserWithPlugins.RegisterPlugin(TestPlugin)
    manager.ParsersManager.RegisterParser(TestParserWithPlugins)
    manager.ParsersManager.RegisterParser(TestParser)

    try:
      expression_invalid_and_valid_names = 'non_existent,test_parser'
      expected_valid_elements = set(['test_parser'])
      expected_invalid_elements = set(['non_existent'])
      valid_elements, invalid_elements = (
          manager.ParsersManager.CheckFilterExpression(
              expression_invalid_and_valid_names))
      self.assertEqual(expected_valid_elements, valid_elements)
      self.assertEqual(expected_invalid_elements, invalid_elements)

      expression_invalid_and_valid_names_with_negation = (
          '!test_parser,!non_existent')
      expected_valid_elements = set(['!test_parser'])
      expected_invalid_elements = set(['!non_existent'])
      valid_elements, invalid_elements = (
          manager.ParsersManager.CheckFilterExpression(
              expression_invalid_and_valid_names_with_negation))
      self.assertEqual(expected_valid_elements, valid_elements)
      self.assertEqual(expected_invalid_elements, invalid_elements)

      expression_with_plugins = (
          '!test_parser_with_plugins/test_plugin,'
          'test_parser_with_plugins/non_existent')
      expected_valid_elements = set(['!test_parser_with_plugins/test_plugin'])
      expected_invalid_elements = set(['test_parser_with_plugins/non_existent'])
      valid_elements, invalid_elements = (
          manager.ParsersManager.CheckFilterExpression(expression_with_plugins))
      self.assertEqual(expected_valid_elements, valid_elements)
      self.assertEqual(expected_invalid_elements, invalid_elements)

      none_expression = None
      all_parser_names = manager.ParsersManager._parser_classes.keys()
      expected_valid_elements = set(all_parser_names)
      expected_invalid_elements = set()
      valid_elements, invalid_elements = (
          manager.ParsersManager.CheckFilterExpression(
              none_expression))
      self.assertEqual(expected_valid_elements, valid_elements)
      self.assertEqual(expected_invalid_elements, invalid_elements)

    # Degister parsers to ensure unrelated tests don't fail.
    finally:
      manager.ParsersManager.DeregisterParser(TestParser)
      manager.ParsersManager.DeregisterParser(TestParserWithPlugins)
      TestParserWithPlugins.DeregisterPlugin(TestPlugin)

  def testGetNamesOfParsersWithPlugins(self):
    """Tests the GetNamesOfParsersWithPlugins function."""
    parsers_names = manager.ParsersManager.GetNamesOfParsersWithPlugins()

    self.assertGreaterEqual(len(parsers_names), 1)

    self.assertIn('winreg', parsers_names)

  def testGetParserAndPluginNames(self):
    """Tests the GetParserAndPluginNames function."""
    TestParserWithPlugins.RegisterPlugin(TestPlugin)
    manager.ParsersManager.RegisterParser(TestParserWithPlugins)
    manager.ParsersManager.RegisterParser(TestParser)

    parser_names = manager.ParsersManager.GetParserAndPluginNames(
        parser_filter_expression='test_parser')
    self.assertEqual(parser_names, ['test_parser'])

    parser_names = manager.ParsersManager.GetParserAndPluginNames(
        parser_filter_expression='!test_parser')
    self.assertNotIn('test_parser', parser_names)

    expected_parser_names = [
        'test_parser_with_plugins',
        'test_parser_with_plugins/test_plugin']
    parser_names = manager.ParsersManager.GetParserAndPluginNames(
        parser_filter_expression='test_parser_with_plugins/test_plugin')
    self.assertEqual(parser_names, expected_parser_names)

    # Test with a parser name, not using plugin names.
    expected_parser_names = [
        'test_parser_with_plugins',
        'test_parser_with_plugins/test_plugin']
    parser_names = manager.ParsersManager.GetParserAndPluginNames(
        parser_filter_expression='test_parser_with_plugins')
    self.assertEqual(parser_names, expected_parser_names)

    TestParserWithPlugins.DeregisterPlugin(TestPlugin)
    manager.ParsersManager.DeregisterParser(TestParserWithPlugins)
    manager.ParsersManager.DeregisterParser(TestParser)

  def testGetParserPluginsInformation(self):
    """Tests the GetParserPluginsInformation function."""
    plugins_information = manager.ParsersManager.GetParserPluginsInformation()

    self.assertGreaterEqual(len(plugins_information), 1)

    available_parser_names = [name for name, _ in plugins_information]
    self.assertIn('olecf_default', available_parser_names)

  def testGetParserObjectByName(self):
    """Tests the GetParserObjectByName function."""
    manager.ParsersManager.RegisterParser(TestParser)

    parser = manager.ParsersManager.GetParserObjectByName(
        'test_parser')
    self.assertIsNotNone(parser)
    self.assertEqual(parser.NAME, 'test_parser')

    parser = manager.ParsersManager.GetParserObjectByName('bogus')
    self.assertIsNone(parser)

    manager.ParsersManager.DeregisterParser(TestParser)

  def testGetParserObjects(self):
    """Tests the GetParserObjects function."""
    TestParserWithPlugins.RegisterPlugin(TestPlugin)
    manager.ParsersManager.RegisterParser(TestParserWithPlugins)
    manager.ParsersManager.RegisterParser(TestParser)

    parser_names = []
    parsers = manager.ParsersManager.GetParserObjects(
        parser_filter_expression='test_parser')
    for _, parser in iter(parsers.items()):
      parser_names.append(parser.NAME)
    self.assertEqual(parser_names, ['test_parser'])

    parser_names = []
    parsers = manager.ParsersManager.GetParserObjects(
        parser_filter_expression='!test_parser')
    for _, parser in iter(parsers.items()):
      parser_names.append(parser.NAME)
    self.assertNotEqual(len(parser_names), 0)
    self.assertNotIn('test_parser', parser_names)

    parser_names = []
    parsers = manager.ParsersManager.GetParserObjects(
        parser_filter_expression='test_parser_with_plugins/test_plugin')
    for _, parser in iter(parsers.items()):
      parser_names.append(parser.NAME)
    self.assertEqual(parser_names, ['test_parser_with_plugins'])

    # Test with a parser name, not using plugin names.
    parser_names = []
    parsers = manager.ParsersManager.GetParserObjects(
        parser_filter_expression='test_parser_with_plugins')
    for _, parser in iter(parsers.items()):
      parser_names.append(parser.NAME)
    self.assertEqual(parser_names, ['test_parser_with_plugins'])

    TestParserWithPlugins.DeregisterPlugin(TestPlugin)
    manager.ParsersManager.DeregisterParser(TestParserWithPlugins)
    manager.ParsersManager.DeregisterParser(TestParser)

  def testGetParsers(self):
    """Tests the _GetParsers function."""
    TestParserWithPlugins.RegisterPlugin(TestPlugin)
    manager.ParsersManager.RegisterParser(TestParserWithPlugins)
    manager.ParsersManager.RegisterParser(TestParser)

    parser_names = []
    for _, parser_class in manager.ParsersManager._GetParsers(
        parser_filter_expression='test_parser'):
      parser_names.append(parser_class.NAME)
    self.assertEqual(parser_names, ['test_parser'])

    parser_names = []
    for _, parser_class in manager.ParsersManager._GetParsers(
        parser_filter_expression='!test_parser'):
      parser_names.append(parser_class.NAME)
    self.assertNotEqual(len(parser_names), 0)
    self.assertNotIn('test_parser', parser_names)

    parser_names = []
    for _, parser_class in manager.ParsersManager._GetParsers(
        parser_filter_expression='test_parser_with_plugins/test_plugin'):
      parser_names.append(parser_class.NAME)
    self.assertEqual(parser_names, ['test_parser_with_plugins'])

    # Test with a parser name, not using plugin names.
    parser_names = []
    for _, parser_class in manager.ParsersManager._GetParsers(
        parser_filter_expression='test_parser_with_plugins'):
      parser_names.append(parser_class.NAME)
    self.assertEqual(parser_names, ['test_parser_with_plugins'])

    TestParserWithPlugins.DeregisterPlugin(TestPlugin)
    manager.ParsersManager.DeregisterParser(TestParserWithPlugins)
    manager.ParsersManager.DeregisterParser(TestParser)

  def testGetParsersInformation(self):
    """Tests the GetParsersInformation function."""
    manager.ParsersManager.RegisterParser(TestParser)

    parsers_information = manager.ParsersManager.GetParsersInformation()

    self.assertGreaterEqual(len(parsers_information), 1)

    available_parser_names = [name for name, _ in parsers_information]
    self.assertIn('test_parser', available_parser_names)

    manager.ParsersManager.DeregisterParser(TestParser)

  def testGetPlugins(self):
    """Tests the GetPlugins function."""
    TestParserWithPlugins.RegisterPlugin(TestPlugin)

    generator = TestParserWithPlugins.GetPlugins()
    plugin_tuples = list(generator)
    self.assertNotEqual(len(plugin_tuples), 0)
    self.assertIsNotNone(plugin_tuples[0])

    TestParserWithPlugins.DeregisterPlugin(TestPlugin)

  def testPluginRegistration(self):
    """Tests the RegisterPlugin and DeregisterPlugin functions."""
    TestParserWithPlugins.RegisterPlugin(TestPlugin)
    self.assertEqual(
        len(TestParserWithPlugins._plugin_classes), 1)

    with self.assertRaises(KeyError):
      TestParserWithPlugins.RegisterPlugin(TestPlugin)

    TestParserWithPlugins.DeregisterPlugin(TestPlugin)
    self.assertEqual(
        len(TestParserWithPlugins._plugin_classes), 0)

  def testGetPluginObjectByName(self):
    """Tests the GetPluginObjectByName function."""
    TestParserWithPlugins.RegisterPlugin(TestPlugin)

    plugin = TestParserWithPlugins.GetPluginObjectByName('test_plugin')
    self.assertIsNotNone(plugin)

    plugin = TestParserWithPlugins.GetPluginObjectByName('bogus')
    self.assertIsNone(plugin)

    TestParserWithPlugins.DeregisterPlugin(TestPlugin)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the 4n6time MySQL output class."""

from __future__ import unicode_literals

import unittest

from plaso import formatters   # pylint: disable=unused-import
from plaso.lib import definitions
from plaso.lib import timelib
from plaso.output import mysql_4n6time

from tests.containers import test_lib as containers_test_lib
from tests.output import fake_mysqldb
from tests.output import test_lib

if mysql_4n6time.MySQLdb is None:
  mysql_4n6time.MySQLdb = fake_mysqldb


class MySQL4n6TimeOutputModuleTest(test_lib.OutputModuleTestCase):
  """Tests for the 4n6time MySQL output class."""

  # pylint: disable=protected-access

  _TEST_EVENTS = [
      {'data_type': 'syslog:line',
       'display_name': 'log/syslog.1',
       'filename': 'log/syslog.1',
       'hostname': 'ubuntu',
       'my_number': 123,
       'some_additional_foo': True,
       'text': (
           'Reporter <CRON> PID: 8442 (pam_unix(cron:session): session '
           'closed for user root)'),
       'timestamp': timelib.Timestamp.CopyFromString(
           '2012-06-27 18:17:01+00:00'),
       'timestamp_desc': definitions.TIME_DESCRIPTION_WRITTEN}]

  def testGetTags(self):
    """Tests the _GetTags function."""
    fake_cursor = fake_mysqldb.FakeMySQLdbCursor()
    fake_cursor.expected_query = 'SELECT DISTINCT tag FROM log2timeline'
    fake_cursor.query_results = [('one',), ('two,three',), ('four',)]

    output_mediator = self._CreateOutputMediator()
    output_module = mysql_4n6time.MySQL4n6TimeOutputModule(output_mediator)
    output_module._cursor = fake_cursor

    expected_tags = ['one', 'two', 'three', 'four']
    tags = output_module._GetTags()
    self.assertEqual(tags, expected_tags)

  def testGetUniqueValues(self):
    """Tests the _GetUniqueValues function."""
    fake_cursor = fake_mysqldb.FakeMySQLdbCursor()
    fake_cursor.expected_query = (
        'SELECT source, COUNT(source) FROM log2timeline GROUP BY source')
    fake_cursor.query_results = [('one', 1), ('two', 2), ('three', 3)]

    output_mediator = self._CreateOutputMediator()
    output_module = mysql_4n6time.MySQL4n6TimeOutputModule(output_mediator)
    output_module._cursor = fake_cursor

    expected_unique_values = {'one': 1, 'two': 2, 'three': 3}
    unique_values = output_module._GetUniqueValues('source')
    self.assertEqual(unique_values, expected_unique_values)

  # TODO: add test for Open and Close

  def testGetSanitizedEventValues(self):
    """Tests the GetSanitizedEventValues function."""
    output_mediator = self._CreateOutputMediator()
    output_module = mysql_4n6time.MySQL4n6TimeOutputModule(output_mediator)

    expected_dict = {
        'type': 'Content Modification Time',
        'host': 'ubuntu',
        'filename': 'log/syslog.1',
        'source': 'LOG',
        'description': '[',
        'datetime': '2012-06-27 18:17:01',
        'inreport': '',
        'source_name': '-',
        'extra': (
            'my_number: 123  some_additional_foo: True  text: '
            'Reporter <CRON> PID: 8442 (pam_unix(cron:session): '
            'session closed for user root) '
        ),
        'tag': '',
        'timezone': 'UTC',
        'inode': '-',
        'reportnotes': '',
        'sourcetype': 'Log File',
        'event_identifier': '-',
        'format': '-',
        'URL': '-',
        'record_number': 0,
        'MACB': 'M...',
        'computer_name': '-',
        'offset': 0,
        'evidence': '-',
        'user_sid': '-',
        'notes': '-',
        'vss_store_number': -1,
        'user': '-'
    }

    event, event_data = containers_test_lib.CreateEventFromValues(
        self._TEST_EVENTS[0])
    event_dict = output_module._GetSanitizedEventValues(event, event_data, None)

    self.assertIsInstance(event_dict, dict)
    self.assertDictContainsSubset(expected_dict, event_dict)

  def testSetCredentials(self):
    """Tests the SetCredentials function."""
    output_mediator = self._CreateOutputMediator()
    output_module = mysql_4n6time.MySQL4n6TimeOutputModule(output_mediator)

    output_module.SetCredentials(password='password', username='username')
    self.assertEqual(output_module._password, 'password')
    self.assertEqual(output_module._user, 'username')

  def testSetDatabaseName(self):
    """Tests the SetDatabaseName function."""
    output_mediator = self._CreateOutputMediator()
    output_module = mysql_4n6time.MySQL4n6TimeOutputModule(output_mediator)

    output_module.SetDatabaseName('database')
    self.assertEqual(output_module._dbname, 'database')

  def testSetServerInformation(self):
    """Tests the SetServerInformation function."""
    output_mediator = self._CreateOutputMediator()
    output_module = mysql_4n6time.MySQL4n6TimeOutputModule(output_mediator)

    output_module.SetServerInformation('127.0.0.1', 3306)
    self.assertEqual(output_module._host, '127.0.0.1')
    self.assertEqual(output_module._port, 3306)

  def testWriteEventBody(self):
    """Tests the WriteEventBody function."""
    fake_cursor = fake_mysqldb.FakeMySQLdbCursor()
    fake_cursor.expected_query = (
        mysql_4n6time.MySQL4n6TimeOutputModule._INSERT_QUERY)

    fake_cursor.expected_query_args = {
        'computer_name': '-',
        'datetime': '2012-06-27 18:17:01',
        'description': '[',
        'event_identifier': '-',
        'event_type': '-',
        'evidence': '-',
        'extra': (
            'my_number: 123  '
            'some_additional_foo: True  '
            'text: Reporter <CRON> PID: 8442 (pam_unix(cron:session): '
            'session closed for user root) '),
        'filename': 'log/syslog.1',
        'format': '-',
        'host': 'ubuntu',
        'inode': '-',
        'inreport': '',
        'MACB': 'M...',
        'notes': '-',
        'offset': 0,
        'record_number': 0,
        'reportnotes': '',
        'source_name': '-',
        'sourcetype': 'Log File',
        'source': 'LOG',
        'tag': '',
        'timezone': 'UTC',
        'type': 'Content Modification Time',
        'URL': '-',
        'user_sid': '-',
        'user': '-',
        'vss_store_number': -1}

    output_mediator = self._CreateOutputMediator()
    output_module = mysql_4n6time.MySQL4n6TimeOutputModule(output_mediator)
    output_module._count = 0
    output_module._cursor = fake_cursor

    event, event_data = containers_test_lib.CreateEventFromValues(
        self._TEST_EVENTS[0])
    output_module.WriteEventBody(event, event_data, None)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""File containing a Windows Registry plugin to parse the USBStor key."""

from __future__ import unicode_literals

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.parsers import logger
from plaso.parsers import winreg
from plaso.parsers.winreg_plugins import interface


class USBStorEventData(events.EventData):
  """USBStor event data attribute container.

  Attributes:
    device_type (str): type of USB device.
    display_name (str): display name of the USB device.
    key_path (str): Windows Registry key path.
    parent_id_prefix (str): parent identifier prefix of the USB device.
    product (str): product of the USB device.
    serial (str): serial number of the USB device.
    revision (str): revision number of the USB device.
    subkey_name (str): name of the Windows Registry subkey.
    vendor (str): vendor of the USB device.
  """

  DATA_TYPE = 'windows:registry:usbstor'

  def __init__(self):
    """Initializes event data."""
    super(USBStorEventData, self).__init__(data_type=self.DATA_TYPE)
    self.device_type = None
    self.display_name = None
    self.key_path = None
    self.parent_id_prefix = None
    self.product = None
    self.revision = None
    self.serial = None
    # TODO: rename subkey_name to something that closer matches its purpose.
    self.subkey_name = None
    self.vendor = None


class USBStorPlugin(interface.WindowsRegistryPlugin):
  """USBStor key plugin.

  Also see:
    http://www.forensicswiki.org/wiki/USB_History_Viewing
  """

  NAME = 'windows_usbstor_devices'
  DESCRIPTION = 'Parser for USB Plug And Play Manager USBStor Registry Key.'

  FILTERS = frozenset([
      interface.WindowsRegistryKeyPathFilter(
          'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Enum\\USBSTOR')])

  def ExtractEvents(self, parser_mediator, registry_key, **kwargs):
    """Extracts events from a Windows Registry key.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
    """
    for subkey in registry_key.GetSubkeys():
      subkey_name = subkey.name

      name_values = subkey_name.split('&')
      number_of_name_values = len(name_values)

      # Normally we expect 4 fields here however that is not always the case.
      if number_of_name_values != 4:
        logger.warning(
            'Expected 4 &-separated values in: {0:s}'.format(subkey_name))

      event_data = USBStorEventData()
      event_data.key_path = registry_key.path
      event_data.subkey_name = subkey_name

      if number_of_name_values >= 1:
        event_data.device_type = name_values[0]
      if number_of_name_values >= 2:
        event_data.vendor = name_values[1]
      if number_of_name_values >= 3:
        event_data.product = name_values[2]
      if number_of_name_values >= 4:
        event_data.revision = name_values[3]

      if subkey.number_of_subkeys == 0:
        # Time last USB device of this class was first inserted.
        event = time_events.DateTimeValuesEvent(
            subkey.last_written_time, definitions.TIME_DESCRIPTION_WRITTEN)
        parser_mediator.ProduceEventWithEventData(event, event_data)
        continue

      for device_key in subkey.GetSubkeys():
        event_data.serial = device_key.name

        friendly_name_value = device_key.GetValueByName('FriendlyName')
        if friendly_name_value:
          event_data.display_name = friendly_name_value.GetDataAsObject()

        # ParentIdPrefix applies to Windows XP Only.
        parent_id_prefix_value = device_key.GetValueByName('ParentIdPrefix')
        if parent_id_prefix_value:
          event_data.parent_id_prefix = parent_id_prefix_value.GetDataAsObject()

        # Time last USB device of this class was first inserted.
        event = time_events.DateTimeValuesEvent(
            subkey.last_written_time, definitions.TIME_DESCRIPTION_WRITTEN)
        parser_mediator.ProduceEventWithEventData(event, event_data)

        # Win7 - Last Connection.
        # Vista/XP - Time of an insert.
        event = time_events.DateTimeValuesEvent(
            device_key.last_written_time, definitions.TIME_DESCRIPTION_WRITTEN)
        parser_mediator.ProduceEventWithEventData(event, event_data)

        device_parameter_key = device_key.GetSubkeyByName('Device Parameters')
        if device_parameter_key:
          event = time_events.DateTimeValuesEvent(
              device_parameter_key.last_written_time,
              definitions.TIME_DESCRIPTION_WRITTEN)
          parser_mediator.ProduceEventWithEventData(event, event_data)

        log_configuration_key = device_key.GetSubkeyByName('LogConf')
        if log_configuration_key:
          event = time_events.DateTimeValuesEvent(
              log_configuration_key.last_written_time,
              definitions.TIME_DESCRIPTION_WRITTEN)
          parser_mediator.ProduceEventWithEventData(event, event_data)

        properties_key = device_key.GetSubkeyByName('Properties')
        if properties_key:
          event = time_events.DateTimeValuesEvent(
              properties_key.last_written_time,
              definitions.TIME_DESCRIPTION_WRITTEN)
          parser_mediator.ProduceEventWithEventData(event, event_data)


winreg.WinRegistryParser.RegisterPlugin(USBStorPlugin)



# -*- coding: utf-8 -*-
"""Warning attribute containers."""

from __future__ import unicode_literals

from plaso.containers import interface
from plaso.containers import manager


# TODO: add AnalysisWarning.


class ExtractionWarning(interface.AttributeContainer):
  """Extraction warning attribute container.

  Extraction warnings are produced by parsers/plugins as well the Plaso engine
  when they encounter situations that should be brought to the users' attention
  but are not events derived from the data being processed.

  Attributes:
    message (str): warning message.
    parser_chain (str): parser chain to which the warning applies.
    path_spec (dfvfs.PathSpec): path specification of the file entry to which
        the warning applies.
  """
  CONTAINER_TYPE = 'extraction_warning'

  def __init__(self, message=None, parser_chain=None, path_spec=None):
    """Initializes an extraction warning.

    Args:
      message (Optional[str]): warning message.
      parser_chain (Optional[str]): parser chain to which the warning applies.
      path_spec (Optional[dfvfs.PathSpec]): path specification of the file entry
          to which the warning applies.
    """
    super(ExtractionWarning, self).__init__()
    self.message = message
    self.parser_chain = parser_chain
    self.path_spec = path_spec


class ExtractionError(ExtractionWarning):
  """Extraction error attribute container.

  This class is provided for backwards compatiblity only, all new code must use
  ExtractionWarning.

  Attributes:
    message (str): warning message.
    parser_chain (str): parser chain to which the warning applies.
    path_spec (dfvfs.PathSpec): path specification of the file entry to which
        the warning applies.
  """
  CONTAINER_TYPE = 'extraction_error'


manager.AttributeContainersManager.RegisterAttributeContainers(
    [ExtractionError, ExtractionWarning])



# -*- coding: utf-8 -*-
"""The text prepend CLI arguments helper."""

from __future__ import unicode_literals

from plaso.cli import tools
from plaso.cli.helpers import interface
from plaso.cli.helpers import manager
from plaso.lib import errors


class TextPrependArgumentsHelper(interface.ArgumentsHelper):
  """Text prepend CLI arguments helper."""

  NAME = 'text_prepend'
  DESCRIPTION = 'Text prepend command line arguments.'

  @classmethod
  def AddArguments(cls, argument_group):
    """Adds command line arguments to an argument group.

    This function takes an argument parser or an argument group object and adds
    to it all the command line arguments this helper supports.

    Args:
      argument_group (argparse._ArgumentGroup|argparse.ArgumentParser):
          argparse group.
    """
    argument_group.add_argument(
        '-t', '--text', dest='text_prepend', action='store', type=str,
        default='', metavar='TEXT', help=(
            'Define a free form text string that is prepended to each path '
            'to make it easier to distinguish one record from another in a '
            'timeline (like c:\\, or host_w_c:\\)'))

  @classmethod
  def ParseOptions(cls, options, configuration_object):
    """Parses and validates options.

    Args:
      options (argparse.Namespace): parser options.
      configuration_object (CLITool): object to be configured by the argument
          helper.

    Raises:
      BadConfigObject: when the configuration object is of the wrong type.
    """
    if not isinstance(configuration_object, tools.CLITool):
      raise errors.BadConfigObject(
          'Configuration object is not an instance of CLITool')

    text_prepend = cls._ParseStringOption(options, 'text_prepend')

    setattr(configuration_object, '_text_prepend', text_prepend)


manager.ArgumentHelperManager.RegisterHelper(TextPrependArgumentsHelper)



# -*- coding: utf-8 -*-
"""The Safari history event formatter."""

from __future__ import unicode_literals

from plaso.formatters import interface
from plaso.formatters import manager


class SafariHistoryFormatter(interface.ConditionalEventFormatter):
  """Formatter for a Safari history event."""

  DATA_TYPE = 'safari:history:visit'

  FORMAT_STRING_PIECES = [
      'Visited: {url}',
      '({title}',
      '- {display_title}',
      ')',
      'Visit Count: {visit_count}']

  SOURCE_LONG = 'Safari History'
  SOURCE_SHORT = 'WEBHIST'


class SafariHistoryFormatterSqlite(interface.ConditionalEventFormatter):
  """Formatter for a Safari history event from Sqlite History.db"""

  DATA_TYPE = 'safari:history:visit_sqlite'

  FORMAT_STRING_PIECES = [
      'URL: {url}',
      'Title: ({title})',
      '[count: {visit_count}]',
      'http_non_get: {was_http_non_get}']


  SOURCE_LONG = 'Safari History'
  SOURCE_SHORT = 'WEBHIST'

manager.FormattersManager.RegisterFormatters([
    SafariHistoryFormatter, SafariHistoryFormatterSqlite])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests that all formatters are imported correctly."""

from __future__ import unicode_literals

import os
import unittest

from tests import test_lib


class FormattersImportTest(test_lib.ImportCheckTestCase):
  """Tests that CLI helper classes are imported correctly."""

  _CLI_HELPERS_PATH = os.path.join(os.getcwd(), 'plaso', 'formatters')
  _IGNORABLE_FILES = frozenset([
      'default.py', 'interface.py', 'logger.py', 'manager.py', 'mediator.py',
      'winevt_rc.py'])

  def testFormattersImported(self):
    """Tests that all parsers are imported."""
    self._AssertFilesImportedInInit(
        self._CLI_HELPERS_PATH, self._IGNORABLE_FILES)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the event formatters interface classes."""

from __future__ import unicode_literals

import unittest

from plaso.formatters import interface
from plaso.formatters import mediator
from plaso.lib import definitions

from tests.containers import test_lib as containers_test_lib
from tests.formatters import test_lib


class BrokenConditionalEventFormatter(interface.ConditionalEventFormatter):
  """An event object for testing the conditional event formatter."""
  DATA_TYPE = 'test:broken_conditional'
  FORMAT_STRING_PIECES = ['{too} {many} formatting placeholders']

  SOURCE_SHORT = 'LOG'
  SOURCE_LONG = 'Some Text File.'


class ConditionalTestEventFormatter(interface.ConditionalEventFormatter):
  """A test conditional event formatter."""

  DATA_TYPE = 'test:event:conditional'

  FORMAT_STRING_PIECES = [
      'Description: {description}',
      'Comment',
      'Value: 0x{numeric:02x}',
      'Optional: {optional}',
      'Text: {text}']

  SOURCE_SHORT = 'LOG'
  SOURCE_LONG = 'Some Text File.'


class WrongEventFormatter(interface.EventFormatter):
  """An event formatter for testing."""
  DATA_TYPE = 'test:wrong'

  FORMAT_STRING = 'This format string does not match {body}.'

  SOURCE_SHORT = 'FILE'
  SOURCE_LONG = 'Weird Log File'


class EventFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the event formatter."""

  _TEST_EVENTS = [
      {'data_type': 'test:event',
       'description': 'this is beyond words',
       'numeric': 12,
       'text': 'but we\'re still trying to say something about the event',
       'timestamp': 1335791207939596,
       'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN}]

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = test_lib.TestEventFormatter()
    self.assertIsNotNone(event_formatter)

  # TODO: add tests for _FormatMessage
  # TODO: add tests for _FormatMessages

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = test_lib.TestEventFormatter()

    expected_attribute_names = ['text']

    attribute_names = event_formatter.GetFormatStringAttributeNames()
    self.assertEqual(sorted(attribute_names), expected_attribute_names)

  def testGetMessages(self):
    """Tests the GetMessages function."""
    formatter_mediator = mediator.FormatterMediator()
    event_formatter = test_lib.TestEventFormatter()

    _, event_data = containers_test_lib.CreateEventFromValues(
        self._TEST_EVENTS[0])

    message, _ = event_formatter.GetMessages(formatter_mediator, event_data)

    self.assertEqual(
        message, 'but we\'re still trying to say something about the event')

  # TODO: add tests for GetSources


class ConditionalEventFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the conditional event formatter."""

  _TEST_EVENTS = [
      {'data_type': 'test:event:conditional',
       'description': 'this is beyond words',
       'numeric': 12,
       'text': 'but we\'re still trying to say something about the event',
       'timestamp': 1335791207939596,
       'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN}]

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = ConditionalTestEventFormatter()
    self.assertIsNotNone(event_formatter)

    with self.assertRaises(RuntimeError):
      BrokenConditionalEventFormatter()

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = ConditionalTestEventFormatter()

    expected_attribute_names = sorted([
        'description', 'numeric', 'optional', 'text'])

    attribute_names = event_formatter.GetFormatStringAttributeNames()
    self.assertEqual(sorted(attribute_names), expected_attribute_names)

  def testGetMessages(self):
    """Tests the GetMessages function."""
    formatter_mediator = mediator.FormatterMediator()
    event_formatter = ConditionalTestEventFormatter()

    _, event_data = containers_test_lib.CreateEventFromValues(
        self._TEST_EVENTS[0])

    message, _ = event_formatter.GetMessages(formatter_mediator, event_data)

    expected_message = (
        'Description: this is beyond words Comment Value: 0x0c '
        'Text: but we\'re still trying to say something about the event')
    self.assertEqual(message, expected_message)

  # TODO: add test for GetSources.


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Event formatter for the MountPoints2 key."""

from __future__ import unicode_literals

from plaso.formatters import interface
from plaso.formatters import manager


class MountPoints2Formatter(interface.ConditionalEventFormatter):
  """Formatter for a Windows Boot Execute event."""

  DATA_TYPE = 'windows:registry:mount_points2'

  FORMAT_STRING_PIECES = [
      '[{key_path}]',
      'Label: {label}',
      'Remote_Server: {server_name}',
      'Share_Name: {share_name}',
      'Type: {type}',
      'Volume: {name}']

  FORMAT_STRING_SHORT_PIECES = [
      '[{key_path}]',
      'Label: {label}',
      'Remote_Server: {server_name}',
      'Share_Name: {share_name}',
      'Type: {type}',
      'Volume: {name}']

  SOURCE_LONG = 'Registry Key'
  SOURCE_SHORT = 'REG'


manager.FormattersManager.RegisterFormatter(MountPoints2Formatter)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Task Scheduler event formatter."""

from __future__ import unicode_literals

import unittest

from plaso.formatters import task_scheduler

from tests.formatters import test_lib


class TaskCacheEventFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the Task Scheduler Cache event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = task_scheduler.TaskCacheEventFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = task_scheduler.TaskCacheEventFormatter()

    expected_attribute_names = [
        'key_path',
        'task_name',
        'task_identifier']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the CLI argument helper interface."""

from __future__ import unicode_literals

import locale
import sys
import unittest

from plaso.lib import errors

from tests.cli import test_lib as cli_test_lib
from tests.cli.helpers import test_lib


class HelperManagerTest(unittest.TestCase):
  """Tests the parsers manager."""

  # pylint: disable=protected-access

  def testParseNumericOption(self):
    """Tests the _ParseNumericOption function."""
    test_helper = test_lib.TestHelper()

    expected_integer = 123
    options = cli_test_lib.TestOptions()
    options.test = expected_integer

    integer = test_helper._ParseNumericOption(options, 'test')
    self.assertEqual(integer, expected_integer)

    options = cli_test_lib.TestOptions()

    integer = test_helper._ParseNumericOption(options, 'test')
    self.assertIsNone(integer)

    integer = test_helper._ParseNumericOption(
        options, 'test', default_value=expected_integer)
    self.assertEqual(integer, expected_integer)

    options = cli_test_lib.TestOptions()
    options.test = b'abc'

    with self.assertRaises(errors.BadConfigOption):
      test_helper._ParseNumericOption(options, 'test')

  def testParseStringOption(self):
    """Tests the _ParseStringOption function."""
    encoding = sys.stdin.encoding

    # Note that sys.stdin.encoding can be None.
    if not encoding:
      encoding = locale.getpreferredencoding()

    test_helper = test_lib.TestHelper()

    expected_string = 'Test Unicode string'
    options = cli_test_lib.TestOptions()
    options.test = expected_string

    string = test_helper._ParseStringOption(options, 'test')
    self.assertEqual(string, expected_string)

    options = cli_test_lib.TestOptions()

    string = test_helper._ParseStringOption(options, 'test')
    self.assertIsNone(string)

    string = test_helper._ParseStringOption(
        options, 'test', default_value=expected_string)
    self.assertEqual(string, expected_string)

    options = cli_test_lib.TestOptions()
    options.test = expected_string.encode(encoding)

    string = test_helper._ParseStringOption(options, 'test')
    self.assertEqual(string, expected_string)

    if encoding and encoding == 'UTF-8':
      options = cli_test_lib.TestOptions()
      options.test = (
          b'\xad\xfd\xab\x73\x99\xc7\xb4\x78\xd0\x8c\x8a\xee\x6d\x6a\xcb\x90')

      with self.assertRaises(errors.BadConfigOption):
        test_helper._ParseStringOption(options, 'test')


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the output module interface."""

from __future__ import unicode_literals

import unittest

from plaso.lib import definitions
from plaso.lib import timelib
from plaso.output import manager

from tests.cli import test_lib as cli_test_lib
from tests.containers import test_lib as containers_test_lib
from tests.output import test_lib


class LinearOutputModuleTest(test_lib.OutputModuleTestCase):
  """Tests the linear output module."""

  _TEST_EVENTS = [
      {'data_type': 'test:event',
       'entry': 'My Event Is Now!',
       'timestamp': timelib.Timestamp.CopyFromString('2012-06-27 18:17:01'),
       'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN},
      {'data_type': 'test:event',
       'entry': 'There is no tomorrow.',
       'timestamp': timelib.Timestamp.CopyFromString('2012-06-27 18:18:23'),
       'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN},
      {'data_type': 'test:event',
       'entry': 'Tomorrow is now.',
       'timestamp': timelib.Timestamp.CopyFromString('2012-06-27 19:11:54'),
       'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN},
      {'data_type': 'test:event',
       'entry': 'This is just some stuff to fill the line.',
       'timestamp': timelib.Timestamp.CopyFromString('2012-06-27 19:12:03'),
       'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN}]

  def testOutput(self):
    """Tests an implementation of output module."""
    output_mediator = self._CreateOutputMediator()
    output_writer = cli_test_lib.TestOutputWriter()
    output_module = test_lib.TestOutputModule(output_mediator)
    output_module.SetOutputWriter(output_writer)
    output_module.WriteHeader()

    for event_values in self._TEST_EVENTS:
      event, event_data = containers_test_lib.CreateEventFromValues(
          event_values)
      output_module.WriteEvent(event, event_data, None)

    output_module.WriteFooter()

    expected_output = (
        '<EventFile>\n'
        '<Event>\n'
        '\t<DateTime>2012-06-27T18:17:01+00:00</DateTime>\n'
        '\t<Entry>My Event Is Now!</Entry>\n'
        '</Event>\n'
        '<Event>\n'
        '\t<DateTime>2012-06-27T18:18:23+00:00</DateTime>\n'
        '\t<Entry>There is no tomorrow.</Entry>\n'
        '</Event>\n'
        '<Event>\n'
        '\t<DateTime>2012-06-27T19:11:54+00:00</DateTime>\n'
        '\t<Entry>Tomorrow is now.</Entry>\n'
        '</Event>\n'
        '<Event>\n'
        '\t<DateTime>2012-06-27T19:12:03+00:00</DateTime>\n'
        '\t<Entry>This is just some stuff to fill the line.</Entry>\n'
        '</Event>\n'
        '</EventFile>\n')

    output = output_writer.ReadOutput()
    self.assertEqual(output, expected_output)

  def testOutputList(self):
    """Test listing up all available registered modules."""
    manager.OutputManager.RegisterOutput(test_lib.TestOutputModule)

    test_output_class = None
    for name, output_class in manager.OutputManager.GetOutputClasses():
      if name == 'test_xml':
        test_output_class = output_class

    expected_description = 'Test output that provides a simple mocked XML.'
    self.assertIsNotNone(test_output_class)
    self.assertEqual(test_output_class.DESCRIPTION, expected_description)

    manager.OutputManager.DeregisterOutput(test_lib.TestOutputModule)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Filter interface."""

from __future__ import unicode_literals

import abc


class FilterObject(object):
  """Filter object interface."""

  def __init__(self):
    """Initializes a filter object."""
    super(FilterObject, self).__init__()
    self._filter_expression = None
    self._matcher = None

  @property
  def fields(self):
    """list[str]: name of the fields."""
    return []

  @property
  def filter_expression(self):
    """object: compiled filter expression or None."""
    if self._filter_expression:
      return self._filter_expression

    return None

  @property
  def filter_name(self):
    """str: name of the filter."""
    return self.__class__.__name__

  @property
  def limit(self):
    """int: row limit."""
    return 0

  @property
  def separator(self):
    """str: output field separator."""
    return ','

  @abc.abstractmethod
  def CompileFilter(self, filter_expression):
    """Compiles the filter expression.

    Args:
      filter_expression (str): filter expression.

    Raises:
      WrongPlugin: if the filter could not be compiled.
    """

  # pylint: disable=unused-argument
  def Match(self, event, event_data, event_tag):
    """Determines if an event matches the filter.

    Args:
      event (EventObject): event.
      event_data (EventData): event data.
      event_tag (EventTag): event tag.

    Returns:
      bool: True if the event matches the filter.
    """
    return False



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Google Analytics cookies."""

from __future__ import unicode_literals

import unittest

from plaso.formatters import ganalytics as _  # pylint: disable=unused-import
from plaso.lib import definitions
from plaso.parsers.cookie_plugins import ganalytics  # pylint: disable=unused-import
from plaso.parsers.sqlite_plugins import chrome_cookies
from plaso.parsers.sqlite_plugins import firefox_cookies

from tests.parsers.sqlite_plugins import test_lib as sqlite_plugins_test_lib


class GoogleAnalyticsPluginTest(sqlite_plugins_test_lib.SQLitePluginTestCase):
  """Tests for the Google Analytics plugin."""

  def _GetAnalyticsCookieEvents(self, storage_writer):
    """Retrieves the analytics cookie events.

    Returns:
      list[EventObject]: analytics cookie events.
    """
    cookies = []
    for event in storage_writer.GetEvents():
      event_data = self._GetEventDataOfEvent(storage_writer, event)
      if event_data.data_type.startswith('cookie:google:analytics'):
        cookies.append(event)
    return cookies

  def testParsingFirefox29CookieDatabase(self):
    """Tests the Process function on a Firefox 29 cookie database file."""
    plugin = firefox_cookies.FirefoxCookiePlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['firefox_cookies.sqlite'], plugin)
    events = self._GetAnalyticsCookieEvents(storage_writer)

    self.assertEqual(storage_writer.number_of_warnings, 0)
    self.assertEqual(len(events), 25)

    event = events[14]

    self.CheckTimestamp(event.timestamp, '2013-10-30 21:56:06.000000')

    event_data = self._GetEventDataOfEvent(storage_writer, event)

    self.assertEqual(
        event_data.utmcct,
        '/frettir/erlent/2013/10/30/maelt_med_kerfisbundnum_hydingum/')
    self.assertEqual(event_data.url, 'http://ads.aha.is/')
    self.assertEqual(event_data.utmcsr, 'mbl.is')

    expected_message = (
        'http://ads.aha.is/ (__utmz) Sessions: 1 Domain Hash: 137167072 '
        'Sources: 1 Last source used to access: mbl.is Ad campaign '
        'information: (referral) Last type of visit: referral Path to '
        'the page of referring link: /frettir/erlent/2013/10/30/'
        'maelt_med_kerfisbundnum_hydingum/')
    expected_short_message = 'http://ads.aha.is/ (__utmz)'

    self._TestGetMessageStrings(
        event_data, expected_message, expected_short_message)

  def testParsingChromeCookieDatabase(self):
    """Test the process function on a Chrome cookie database."""
    plugin = chrome_cookies.ChromeCookiePlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['cookies.db'], plugin)
    events = self._GetAnalyticsCookieEvents(storage_writer)

    self.assertEqual(storage_writer.number_of_warnings, 1)

    # The cookie database contains 560 entries in total. Out of them
    # there are 75 events created by the Google Analytics plugin.
    self.assertEqual(len(events), 75)
    # Check few "random" events to verify.

    # Check an UTMZ Google Analytics event.
    event = events[39]

    event_data = self._GetEventDataOfEvent(storage_writer, event)
    self.assertEqual(event_data.utmctr, 'enders game')
    self.assertEqual(event_data.domain_hash, '68898382')
    self.assertEqual(event_data.sessions, 1)

    expected_message = (
        'http://imdb.com/ (__utmz) Sessions: 1 Domain Hash: 68898382 '
        'Sources: 1 Last source used to access: google Ad campaign '
        'information: (organic) Last type of visit: organic Keywords '
        'used to find site: enders game')
    expected_short_message = 'http://imdb.com/ (__utmz)'

    self._TestGetMessageStrings(
        event_data, expected_message, expected_short_message)

    # Check the UTMA Google Analytics event.
    event = events[41]

    self.CheckTimestamp(event.timestamp, '2012-03-22 01:55:29.000000')
    self.assertEqual(event.timestamp_desc, 'Analytics Previous Time')

    event_data = self._GetEventDataOfEvent(storage_writer, event)
    self.assertEqual(event_data.cookie_name, '__utma')
    self.assertEqual(event_data.visitor_id, '1827102436')
    self.assertEqual(event_data.sessions, 2)

    expected_message = (
        'http://assets.tumblr.com/ (__utma) '
        'Sessions: 2 '
        'Domain Hash: 151488169 '
        'Visitor ID: 1827102436')
    expected_short_message = 'http://assets.tumblr.com/ (__utma)'

    self._TestGetMessageStrings(
        event_data, expected_message, expected_short_message)

    # Check the UTMB Google Analytics event.
    event = events[34]

    self.CheckTimestamp(event.timestamp, '2012-03-22 01:48:30.000000')
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_LAST_VISITED)

    event_data = self._GetEventDataOfEvent(storage_writer, event)
    self.assertEqual(event_data.cookie_name, '__utmb')
    self.assertEqual(event_data.domain_hash, '154523900')
    self.assertEqual(event_data.pages_viewed, 1)

    expected_message = (
        'http://upressonline.com/ (__utmb) Pages Viewed: 1 Domain Hash: '
        '154523900')
    expected_short_message = 'http://upressonline.com/ (__utmb)'

    self._TestGetMessageStrings(
        event_data, expected_message, expected_short_message)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""This file contains the Property List (Plist) Parser.

Plaso's engine calls PlistParser when it encounters Plist files to be processed.
"""

from __future__ import unicode_literals


import biplist

from plaso.lib import errors
from plaso.lib import specification
from plaso.parsers import interface
from plaso.parsers import logger
from plaso.parsers import manager


class PlistParser(interface.FileObjectParser):
  """Parses binary and text plist plist files.

  The Plaso engine calls parsers by their Parse() method. This parser's
  Parse() has GetTopLevel() which deserializes plist files using the biplist
  library and calls plugins (PlistPlugin) registered through the
  interface by their Process() to produce event objects.

  Plugins are how this parser understands the content inside a plist file,
  each plugin holds logic specific to a particular plist file. See the
  interface and plist_plugins/ directory for examples of how plist plugins are
  implemented.
  """

  NAME = 'plist'
  DESCRIPTION = 'Parser for binary and text plist files.'

  _plugin_classes = {}

  @classmethod
  def GetFormatSpecification(cls):
    """Retrieves the format specification.

    Returns:
      FormatSpecification: a format specification or None if not available.
    """
    format_specification = specification.FormatSpecification(cls.NAME)
    format_specification.AddNewSignature(b'bplist', offset=0)
    return format_specification

  def GetTopLevel(self, file_object):
    """Returns the deserialized content of a plist as a dictionary object.

    Args:
      file_object (dfvfs.FileIO): a file-like object to parse.

    Returns:
      dict[str, object]: contents of the plist.

    Raises:
      UnableToParseFile: when the file cannot be parsed.
    """
    try:
      top_level_object = biplist.readPlist(file_object)

    except (biplist.InvalidPlistException,
            biplist.NotBinaryPlistException) as exception:
      raise errors.UnableToParseFile(
          'Unable to parse plist with error: {0!s}'.format(exception))

    return top_level_object

  def ParseFileObject(self, parser_mediator, file_object):
    """Parses a plist file-like object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_object (dfvfs.FileIO): a file-like object.

    Raises:
      UnableToParseFile: when the file cannot be parsed.
    """
    filename = parser_mediator.GetFilename()
    file_size = file_object.get_size()

    if file_size <= 0:
      raise errors.UnableToParseFile(
          'File size: {0:d} bytes is less equal 0.'.format(file_size))

    # 50MB is 10x larger than any plist seen to date.
    if file_size > 50000000:
      raise errors.UnableToParseFile(
          'File size: {0:d} bytes is larger than 50 MB.'.format(file_size))

    top_level_object = self.GetTopLevel(file_object)
    if not top_level_object:
      raise errors.UnableToParseFile(
          'Unable to parse: {0:s} skipping.'.format(filename))

    # TODO: add a parser filter.
    matching_plugin = None
    for plugin in self._plugins:
      try:
        plugin.UpdateChainAndProcess(
            parser_mediator, plist_name=filename, top_level=top_level_object)
        matching_plugin = plugin

      except errors.WrongPlistPlugin as exception:
        logger.debug('Wrong plugin: {0:s} for: {1:s}'.format(
            exception.args[0], exception.args[1]))

    if not matching_plugin and self._default_plugin:
      self._default_plugin.UpdateChainAndProcess(
          parser_mediator, plist_name=filename, top_level=top_level_object)


manager.ParsersManager.RegisterParser(PlistParser)



# -*- coding: utf-8 -*-
""""Windows Registry plugin for SAM Users Account information."""

from __future__ import unicode_literals

from dfdatetime import filetime as dfdatetime_filetime

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.lib import errors
from plaso.parsers import winreg
from plaso.parsers.winreg_plugins import dtfabric_plugin
from plaso.parsers.winreg_plugins import interface


class SAMUsersWindowsRegistryEventData(events.EventData):
  """Class that defines SAM users Windows Registry event data.

  Attributes:
    account_rid (int): account relative identifier (RID).
    comments (str): comments.
    fullname (str): full name.
    key_path (str): Windows Registry key path.
    login_count (int): login count.
    username (str): a string containing the username.
  """
  DATA_TYPE = 'windows:registry:sam_users'

  def __init__(self):
    """Initializes event data."""
    super(SAMUsersWindowsRegistryEventData, self).__init__(
        data_type=self.DATA_TYPE)
    self.account_rid = None
    self.comments = None
    self.fullname = None
    self.key_path = None
    self.login_count = None
    self.username = None


class SAMUsersWindowsRegistryPlugin(
    dtfabric_plugin.DtFabricBaseWindowsRegistryPlugin):
  """Windows Registry plugin for SAM Users Account information."""

  NAME = 'windows_sam_users'
  DESCRIPTION = 'Parser for SAM Users and Names Registry keys.'

  FILTERS = frozenset([
      interface.WindowsRegistryKeyPathFilter(
          'HKEY_LOCAL_MACHINE\\SAM\\SAM\\Domains\\Account\\Users')])

  _DEFINITION_FILE = 'sam_users.yaml'

  _V_VALUE_STRINGS_OFFSET = 0xcc

  def _ParseFValue(self, registry_key):
    """Parses an F value.

    Args:
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.

    Returns:
      f_value: F value stored in the Windows Registry key.

    Raises:
      ParseError: if the Windows Registry key does not contain an F value or
          F value cannot be parsed.
    """
    registry_value = registry_key.GetValueByName('F')
    if not registry_value:
      raise errors.ParseError(
          'missing value: "F" in Windows Registry key: {0:s}.'.format(
              registry_key.name))

    f_value_map = self._GetDataTypeMap('f_value')

    try:
      return self._ReadStructureFromByteStream(
          registry_value.data, 0, f_value_map)
    except (ValueError, errors.ParseError) as exception:
      raise errors.ParseError(exception)

  def _ParseVValueString(
      self, parser_mediator, data, user_information_descriptor):
    """Parses a V value string.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      data (bytes): Windows Registry V value data.
      user_information_descriptor (user_information_descriptor): V value
          user information descriptor.

    Returns:
      str: string value stored in the Windows Registry V value data.
    """
    data_start_offset = (
        user_information_descriptor.offset + self._V_VALUE_STRINGS_OFFSET)
    data_end_offset = data_start_offset + user_information_descriptor.size
    descriptor_data = data[data_start_offset:data_end_offset]

    try:
      username = descriptor_data.decode('utf-16-le')
    except (UnicodeDecodeError, UnicodeEncodeError) as exception:
      username = descriptor_data.decode('utf-16-le', errors='replace')
      parser_mediator.ProduceExtractionWarning((
          'unable to decode V value string with error: {0!s}. Characters '
          'that cannot be decoded will be replaced with "?" or '
          '"\\ufffd".').format(exception))

    return username

  def ExtractEvents(self, parser_mediator, registry_key, **kwargs):
    """Extracts events from a Windows Registry key.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
    """
    names_key = registry_key.GetSubkeyByName('Names')
    if not names_key:
      parser_mediator.ProduceExtractionWarning('missing subkey: Names.')
      return

    last_written_time_per_username = {
        registry_value.name: registry_value.last_written_time
        for registry_value in names_key.GetSubkeys()}

    for subkey in registry_key.GetSubkeys():
      if subkey.name == 'Names':
        continue

      try:
        f_value = self._ParseFValue(subkey)
      except errors.ParseError as exception:
        parser_mediator.ProduceExtractionWarning(
            'unable to parse F value with error: {0!s}'.format(exception))
        continue

      registry_value = subkey.GetValueByName('V')
      if not registry_value:
        parser_mediator.ProduceExtractionWarning(
            'missing Registry value: "V" in subkey: {0:s}.'.format(
                subkey.name))
        continue

      v_value_map = self._GetDataTypeMap('v_value')

      try:
        v_value = self._ReadStructureFromByteStream(
            registry_value.data, 0, v_value_map)
      except (ValueError, errors.ParseError) as exception:
        parser_mediator.ProduceExtractionWarning(
            'unable to parse V value with error: {0!s}'.format(exception))
        continue

      username = self._ParseVValueString(
          parser_mediator, registry_value.data, v_value[1])

      fullname = self._ParseVValueString(
          parser_mediator, registry_value.data, v_value[2])

      comments = self._ParseVValueString(
          parser_mediator, registry_value.data, v_value[3])

      last_written_time = last_written_time_per_username.get(username, None)

      # TODO: check if subkey.name == f_value.rid

      event_data = SAMUsersWindowsRegistryEventData()
      event_data.account_rid = f_value.rid
      event_data.comments = comments
      event_data.fullname = fullname
      event_data.key_path = registry_key.path
      event_data.login_count = f_value.number_of_logons
      event_data.username = username

      event = time_events.DateTimeValuesEvent(
          last_written_time, definitions.TIME_DESCRIPTION_WRITTEN)
      parser_mediator.ProduceEventWithEventData(event, event_data)

      if f_value.last_login_time != 0:
        date_time = dfdatetime_filetime.Filetime(
            timestamp=f_value.last_login_time)
        event = time_events.DateTimeValuesEvent(
            date_time, definitions.TIME_DESCRIPTION_LAST_LOGIN)
        parser_mediator.ProduceEventWithEventData(event, event_data)

      if f_value.last_password_set_time != 0:
        date_time = dfdatetime_filetime.Filetime(
            timestamp=f_value.last_password_set_time)
        event = time_events.DateTimeValuesEvent(
            date_time, definitions.TIME_DESCRIPTION_LAST_PASSWORD_RESET)
        parser_mediator.ProduceEventWithEventData(event, event_data)


winreg.WinRegistryParser.RegisterPlugin(SAMUsersWindowsRegistryPlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Advanced Packaging Tool (APT) History log event formatter."""

from __future__ import unicode_literals

import unittest

from plaso.formatters import apt_history

from tests.formatters import test_lib


class AptHistoryLogFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the APT History log event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = apt_history.APTHistoryLogFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = apt_history.APTHistoryLogFormatter()

    expected_attribute_names = [
        'packages',
        'command',
        'error',
        'requester']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Windows Registry plugin to parse the Background Activity Moderator keys."""

from __future__ import unicode_literals

from dfdatetime import filetime as dfdatetime_filetime

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.lib import errors
from plaso.parsers import winreg
from plaso.parsers.winreg_plugins import interface
from plaso.parsers.winreg_plugins import dtfabric_plugin


class BackgroundActivityModeratorEventData(events.EventData):
  """Background Activity Moderator event data.

  Attributes:
    binary_path (str): binary executed.
    user_sid (str): user SID associated with entry.
  """

  DATA_TYPE = 'windows:registry:bam'

  def __init__(self):
    """Initializes event data."""
    super(
        BackgroundActivityModeratorEventData,
        self).__init__(data_type=self.DATA_TYPE)
    self.binary_path = None
    self.user_sid = None


class BackgroundActivityModeratorWindowsRegistryPlugin(
    dtfabric_plugin.DtFabricBaseWindowsRegistryPlugin):
  """Background Activity Moderator data Windows Registry plugin."""

  NAME = 'bam'
  DESCRIPTION = 'Parser for Background Activity Moderator Registry data.'

  FILTERS = frozenset([
      interface.WindowsRegistryKeyPathFilter(
          'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Services\\bam'
          '\\UserSettings'),
      interface.WindowsRegistryKeyPathFilter(
          'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Services\\bam'
          '\\State\\UserSettings')])

  _DEFINITION_FILE = 'filetime.yaml'

  def _ParseValue(self, registry_value):
    """Parses the registry value.

    Args:
      registry_value (bytes): value data.

    Returns:
      int: timestamp.

    Raises:
      ParseError: if the value data could not be parsed.
    """
    try:
      timestamp = self._ReadStructureFromByteStream(
          registry_value, 0, self._GetDataTypeMap('filetime'))
    except (ValueError, errors.ParseError) as exception:
      raise errors.ParseError(
          'Unable to parse timestamp with error: {0!s}'.format(
              exception))

    return timestamp

  def ExtractEvents(self, parser_mediator, registry_key, **kwargs):
    """Extracts events from a Windows Registry key.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.

    Raises:
      ParseError: if the value data could not be parsed.
    """
    sid_keys = registry_key.GetSubkeys()
    if not sid_keys:
      return

    for sid_key in sid_keys:
      for value in sid_key.GetValues():
        if not value.name == 'Version' and not value.name == 'SequenceNumber':
          timestamp = self._ParseValue(value.data)

          if timestamp:
            event_data = BackgroundActivityModeratorEventData()
            event_data.binary_path = value.name
            event_data.user_sid = sid_key.name

            date_time = dfdatetime_filetime.Filetime(timestamp=timestamp)

            event = time_events.DateTimeValuesEvent(
                date_time, definitions.TIME_DESCRIPTION_LAST_RUN)
            parser_mediator.ProduceEventWithEventData(event, event_data)


winreg.WinRegistryParser.RegisterPlugin(
    BackgroundActivityModeratorWindowsRegistryPlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the server configuration CLI arguments helper."""

from __future__ import unicode_literals

import argparse
import unittest

from plaso.cli.helpers import server_config
from plaso.lib import errors

from tests.cli import test_lib as cli_test_lib


class MockOutputModule(object):
  """Mock output module for testing.

  Attributes:
    hostname (str): hostname or IP address of the database server.
    port (int): port number of the database server.
  """

  def __init__(self):
    """Initializes a mock output module."""
    super(MockOutputModule, self).__init__()
    self.hostname = None
    self.port = None

  def SetServerInformation(self, server, port):
    """Sets the server information.

    Args:
      server (str): hostname or IP address of the database server.
      port (int): port number of the database server.
    """
    self.hostname = server
    self.port = port


class ServerArgumentsHelperTest(cli_test_lib.CLIToolTestCase):
  """Tests the server configuration CLI arguments helper."""

  # pylint: disable=no-member,protected-access

  _EXPECTED_OUTPUT = """\
usage: cli_helper.py [--server HOSTNAME] [--port PORT]

Test argument parser.

optional arguments:
  --port PORT        The port number of the server.
  --server HOSTNAME  The hostname or server IP address of the server.
"""

  def testAddArguments(self):
    """Tests the AddArguments function."""
    argument_parser = argparse.ArgumentParser(
        prog='cli_helper.py',
        description='Test argument parser.', add_help=False,
        formatter_class=cli_test_lib.SortedArgumentsHelpFormatter)

    server_config.ServerArgumentsHelper.AddArguments(argument_parser)

    output = self._RunArgparseFormatHelp(argument_parser)
    self.assertEqual(output, self._EXPECTED_OUTPUT)

  def testParseOptions(self):
    """Tests the ParseOptions function."""
    options = cli_test_lib.TestOptions()

    output_module = MockOutputModule()
    server_config.ServerArgumentsHelper.ParseOptions(options, output_module)

    with self.assertRaises(errors.BadConfigObject):
      server_config.ServerArgumentsHelper.ParseOptions(options, None)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for Twitter on Android database event formatter."""

from __future__ import unicode_literals

import unittest

from plaso.formatters import twitter_android
from tests.formatters import test_lib


class TwitterAndroidContactFormatterTest(test_lib.EventFormatterTestCase):
  """Tests the Twitter on Android contact event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = twitter_android.TwitterAndroidContactFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = twitter_android.TwitterAndroidContactFormatter()

    expected_attribute_names = [
        'username',
        'image_url',
        'name',
        'location',
        'description',
        'web_url',
        'followers',
        'friend',
        'statuses'
    ]

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)


class TwitterAndroidStatusFormatterTest(test_lib.EventFormatterTestCase):
  """Tests the Twitter on Android status event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = twitter_android.TwitterAndroidStatusFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = twitter_android.TwitterAndroidStatusFormatter()

    expected_attribute_names = [
        'username',
        'content',
        'favorited',
        'retweeted'
    ]

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)


class TwitterAndroidSearchFormatterTest(test_lib.EventFormatterTestCase):
  """Tests the Twitter on Android search event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = twitter_android.TwitterAndroidSearchFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = twitter_android.TwitterAndroidSearchFormatter()

    expected_attribute_names = [
        'name',
        'search_query'
    ]

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Fake implementation of MySQLdb module for testing."""

from __future__ import unicode_literals


class Error(object):
  """Fake implementation of MySQLdb Error class for testing."""


class FakeMySQLdbConnection(object):
  """Fake implementation of MySQLdb Connection class for testing."""

  # Note: that the following functions do not follow the style guide
  # because they are part of the MySQL database connection interface.
  # pylint: disable=invalid-name

  def close(self):
    """Closes the connection."""
    return

  def commit(self):
    """Commits changes to the database."""
    return

  def cursor(self):
    """Retrieves a database cursor.

    Returns:
      FakeMySQLCursor: cursor.
    """
    return FakeMySQLdbCursor()

  # pylint: disable=unused-argument
  def set_character_set(self, character_set):
    """Sets the character set.

    Args:
      character_set (str): character set.
    """
    return


class FakeMySQLdbCursor(object):
  """Fake implementation of MySQLdb Cursor class for testing.

  Attributes:
    expected_query (str): query expected to be passed to the execute method.
    query_results (list[object]): rows to return as results of the query.
  """

  def __init__(self):
    """Initializes the cursor."""
    super(FakeMySQLdbCursor, self).__init__()
    self._result_index = 0
    self.expected_query = None
    self.expected_query_args = None
    self.query_results = []

  # Note: that the following functions do not follow the style guide
  # because they are part of the MySQL database cursor interface.
  # pylint: disable=invalid-name

  def close(self):
    """Closes the cursor."""
    return

  def execute(self, query, args=None):
    """Executes the query.

    Args:
      query (str): SQL query.
      args (Optional[object]): sequence or mapping of the parameters to use
          with the query.

    Raises:
      ValueError: if the query or query arguments do not match the expected
          values.
    """
    if self.expected_query is not None and self.expected_query != query:
      raise ValueError('Query mismatch.')

    if (self.expected_query_args is not None and
        self.expected_query_args != args):
      raise ValueError('Query arguments mismatch.')

    self._result_index = 0

  def fetchone(self):
    """Fetches a single row of the results returned by execute.

    Returns:
      object: row or None.
    """
    if (not self.query_results or self._result_index < 0 or
        self._result_index >= len(self.query_results)):
      return None

    row = self.query_results[self._result_index]
    self._result_index += 1
    return row


# Note: that the following function does not follow the style guide
# because they are part of the MySQL database module interface.
# pylint: disable=invalid-name,unused-argument
def connect(hostname, username, password, database_name):
  """Connects to the MySQL database server.

  Args:
    hostname (str): hostname of the server.
    username (str): username to use to connect to the server.
    password (str): password to use to connect to the server.
    database_name (str): name of the database on the server.

  Returns:
    FakeMySQLdbConnection: connection
  """
  return FakeMySQLdbConnection()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the OpenXML event formatter."""

from __future__ import unicode_literals

import unittest

from plaso.formatters import oxml

from tests.formatters import test_lib


class OpenXMLParserFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the OXML event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = oxml.OpenXMLParserFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = oxml.OpenXMLParserFormatter()

    expected_attribute_names = [
        'creating_app',
        'app_version',
        'title',
        'subject',
        'last_saved_by',
        'author',
        'total_edit_time',
        'keywords',
        'comments',
        'revision_number',
        'template',
        'number_of_pages',
        'number_of_words',
        'number_of_characters',
        'number_of_characters_with_spaces',
        'number_of_lines',
        'company',
        'manager',
        'shared',
        'security',
        'hyperlinks_changed',
        'links_up_to_date',
        'scale_crop',
        'dig_sig',
        'slides',
        'hidden_slides',
        'presentation_format',
        'mm_clips',
        'notes']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Script to plot tasks from profiling data.

This script requires the matplotlib and numpy Python modules.
"""

from __future__ import print_function
# mathplotlib does not support Unicode strings a column names.
# from __future__ import unicode_literals

import argparse
import glob
import os
import sys

import numpy  # pylint: disable=import-error

from matplotlib import pyplot  # pylint: disable=import-error


class TaskMeasurements(object):
  """Measurements of a task.

  Attributes:
    completed_time (float): time when the task was completed by the foreman.
    created_time (float): time when the task was created by the foreman.
    merging_duration (float): time it took the foreman to merge the task.
    merging_time (float): time when the task was started to be merged by
        the foreman.
    pending_merge (float): time when the task was scheduled to be merged by
        the foreman.
    processed_time (float): time when the task was processed according to
        the foreman.
    processing_duration (float): time it took the worker to process the task.
    processing_time (float): time when the task started to be processed by
        the worker.
    scheduled_time (float): time when the task was scheduled onto the task
        queue by the foreman.
  """

  def __init__(self):
    """Initializes a task measurement."""
    super(TaskMeasurements, self).__init__()
    self.completed_time = None
    self.created_time = None
    self.merging_duration = None
    self.merging_time = None
    self.pending_merge_time = None
    self.processed_time = None
    self.processing_duration = None
    self.processing_time = None
    self.scheduled_time = None


def Main():
  """The main program function.

  Returns:
    bool: True if successful or False if not.
  """
  argument_parser = argparse.ArgumentParser(description=(
      'Plots memory usage from profiling data.'))

  argument_parser.add_argument(
      '--output', dest='output_file', type=str, help=(
          'path of the output file to write the graph to instead of using '
          'interactive mode. The output format deduced from the extension '
          'of the filename.'))

  argument_parser.add_argument(
      'profile_path', type=str, help=(
          'path to the directory containing the profiling data.'))

  options = argument_parser.parse_args()

  if not os.path.isdir(options.profile_path):
    print('No such directory: {0:s}'.format(options.profile_path))
    return False

  names = ['time', 'identifier', 'status']

  measurements = {}

  glob_expression = os.path.join(options.profile_path, 'tasks-*.csv.gz')
  for csv_file_name in glob.glob(glob_expression):
    data = numpy.genfromtxt(
        csv_file_name, delimiter='\t', dtype=None, encoding='utf-8',
        names=names, skip_header=1)

    label = os.path.basename(csv_file_name)
    label = label.replace('tasks-', '').replace('.csv.gz', '')

    for time, identifier, status in data:
      if identifier not in measurements:
        measurements[identifier] = TaskMeasurements()

      task_measurement = measurements[identifier]

      if status == 'completed':
        task_measurement.completed_time = time
        task_measurement.merging_duration = time - task_measurement.merging_time

      elif status == 'created':
        task_measurement.created_time = time

      # TODO: add support for:
      # elif status == 'merge_on_hold':
      # elif status == 'merge_resumed':

      elif status == 'merge_started':
        task_measurement.merging_time = time

      elif status == 'pending_merge':
        task_measurement.pending_merge_time = time

      elif status == 'processed':
        task_measurement.processed_time = time

      elif status == 'processing_started':
        task_measurement.processing_time = time

      elif status == 'processing_completed':
        task_measurement.processing_duration = (
            time - task_measurement.processing_time)

      elif status == 'scheduled':
        task_measurement.scheduled_time = time

  before_pending_merge_duration = {}
  before_queued_duration = {}
  merging_duration = {}
  pending_merge_duration = {}
  processing_duration = {}
  queued_duration = {}

  for identifier, task_measurement in measurements.items():
    before_pending_merge_duration[task_measurement.scheduled_time] = (
        task_measurement.pending_merge_time - (
            task_measurement.processing_time +
            task_measurement.processing_duration))

    before_queued_duration[task_measurement.scheduled_time] = (
        task_measurement.scheduled_time - task_measurement.created_time)

    merging_duration[task_measurement.merging_time] = (
        task_measurement.merging_duration)

    pending_merge_duration[task_measurement.processing_time] = (
        task_measurement.merging_time - task_measurement.pending_merge_time)

    processing_duration[task_measurement.processing_time] = (
        task_measurement.processing_duration)

    queued_duration[task_measurement.scheduled_time] = (
        task_measurement.processing_time - task_measurement.scheduled_time)

  keys = sorted(before_pending_merge_duration.keys())
  values = [before_pending_merge_duration[key] for key in keys]
  pyplot.plot(keys, values, label='Before pending merge')

  keys = sorted(before_queued_duration.keys())
  values = [before_queued_duration[key] for key in keys]
  pyplot.plot(keys, values, label='Before queued')

  keys = sorted(merging_duration.keys())
  values = [merging_duration[key] for key in keys]
  pyplot.plot(keys, values, label='Merging')

  keys = sorted(pending_merge_duration.keys())
  values = [pending_merge_duration[key] for key in keys]
  pyplot.plot(keys, values, label='Pending merge')

  keys = sorted(processing_duration.keys())
  values = [processing_duration[key] for key in keys]
  pyplot.plot(keys, values, label='Processing')

  keys = sorted(queued_duration.keys())
  values = [queued_duration[key] for key in keys]
  pyplot.plot(keys, values, label='Queued')

  pyplot.title('Task status duration')

  pyplot.xlabel('Time')
  pyplot.xscale('linear')

  pyplot.ylabel('Duration')
  pyplot.yscale('linear')

  pyplot.legend()

  if options.output_file:
    pyplot.savefig(options.output_file)
  else:
    pyplot.show()

  return True


if __name__ == '__main__':
  if not Main():
    sys.exit(1)
  else:
    sys.exit(0)



# -*- coding: utf-8 -*-
"""The Windows XML EventLog (EVTX) file event formatter."""

from __future__ import unicode_literals

from plaso.formatters import interface
from plaso.formatters import manager
from plaso.lib import errors


class WinEVTXFormatter(interface.ConditionalEventFormatter):
  """Formatter for a Windows XML EventLog (EVTX) record event."""

  DATA_TYPE = 'windows:evtx:record'

  FORMAT_STRING_PIECES = [
      '[{event_identifier} /',
      '0x{event_identifier:04x}]',
      'Source Name: {source_name}',
      'Message string: {message_string}',
      'Strings: {strings}',
      'Computer Name: {computer_name}',
      'Record Number: {record_number}',
      'Event Level: {event_level}']

  FORMAT_STRING_SHORT_PIECES = [
      '[{event_identifier} /',
      '0x{event_identifier:04x}]',
      'Strings: {strings}']

  SOURCE_LONG = 'WinEVTX'
  SOURCE_SHORT = 'EVT'

  def GetMessages(self, formatter_mediator, event_data):
    """Determines the formatted message strings for the event data.

    Args:
      formatter_mediator (FormatterMediator): mediates the interactions between
          formatters and other components, such as storage and Windows EventLog
          resources.
      event_data (EventData): event data.

    Returns:
      tuple(str, str): formatted message string and short message string.

    Raises:
      WrongFormatter: if the event data cannot be formatted by the formatter.
    """
    if self.DATA_TYPE != event_data.data_type:
      raise errors.WrongFormatter('Unsupported data type: {0:s}.'.format(
          event_data.data_type))

    event_values = event_data.CopyToDict()

    source_name = event_values.get('source_name', None)
    message_identifier = event_values.get('message_identifier', None)
    strings = event_values.get('strings', [])
    if source_name and message_identifier:
      message_string = formatter_mediator.GetWindowsEventMessage(
          source_name, message_identifier)
      if message_string:
        try:
          event_values['message_string'] = message_string.format(*strings)
        except IndexError:
          pass

    message_strings = []
    for string in strings:
      if string:
        message_strings.append('\'{0:s}\''.format(string))
    message_string = ', '.join(message_strings)
    event_values['strings'] = '[{0:s}]'.format(message_string)

    return self._ConditionalFormatMessages(event_values)


manager.FormattersManager.RegisterFormatter(WinEVTXFormatter)



# -*- coding: utf-8 -*-
"""Parser for Windows Restore Point (rp.log) files."""

from __future__ import unicode_literals

from dfdatetime import filetime as dfdatetime_filetime
from dfdatetime import semantic_time as dfdatetime_semantic_time

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.lib import errors
from plaso.parsers import dtfabric_parser
from plaso.parsers import interface
from plaso.parsers import manager


class RestorePointEventData(events.EventData):
  """Windows Restore Point event data.

  Attributes:
    description (str): description.
    restore_point_event_type (str): restore point event type.
    restore_point_type (str): restore point type.
    sequence_number (str): sequence number.
  """

  DATA_TYPE = 'windows:restore_point:info'

  def __init__(self):
    """Initializes Windows Recycle Bin event data."""
    super(RestorePointEventData, self).__init__(data_type=self.DATA_TYPE)
    self.description = None
    self.restore_point_event_type = None
    self.restore_point_type = None
    self.sequence_number = None


class RestorePointLogParser(dtfabric_parser.DtFabricBaseParser):
  """A parser for Windows Restore Point (rp.log) files."""

  NAME = 'rplog'
  DESCRIPTION = 'Parser for Windows Restore Point (rp.log) files.'

  FILTERS = frozenset([
      interface.FileNameFileEntryFilter('rp.log')])

  _DEFINITION_FILE = 'winrestore.yaml'

  def ParseFileObject(self, parser_mediator, file_object):
    """Parses a Windows Restore Point (rp.log) log file-like object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_object (dfvfs.FileIO): file-like object.

    Raises:
      UnableToParseFile: when the file cannot be parsed.
    """
    file_size = file_object.get_size()

    file_header_map = self._GetDataTypeMap('rp_log_file_header')

    try:
      file_header, _ = self._ReadStructureFromFileObject(
          file_object, 0, file_header_map)
    except (ValueError, errors.ParseError) as exception:
      raise errors.UnableToParseFile(
          'Unable to parse file header with error: {0!s}'.format(
              exception))

    file_footer_map = self._GetDataTypeMap('rp_log_file_footer')

    file_footer_offset = file_size - file_footer_map.GetByteSize()

    try:
      file_footer, _ = self._ReadStructureFromFileObject(
          file_object, file_footer_offset, file_footer_map)
    except (ValueError, errors.ParseError) as exception:
      parser_mediator.ProduceExtractionWarning(
          'unable to parse file footer with error: {0!s}'.format(exception))
      return

    # The description in the file header includes the end-of-string character
    # that we need to strip off.
    description = file_header.description.rstrip('\0')

    if file_footer.creation_time == 0:
      date_time = dfdatetime_semantic_time.SemanticTime('Not set')
    else:
      date_time = dfdatetime_filetime.Filetime(
          timestamp=file_footer.creation_time)

    event_data = RestorePointEventData()
    event_data.description = description
    event_data.restore_point_event_type = file_header.event_type
    event_data.restore_point_type = file_header.restore_point_type
    event_data.sequence_number = file_header.sequence_number

    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_CREATION)
    parser_mediator.ProduceEventWithEventData(event, event_data)


manager.ParsersManager.RegisterParser(RestorePointLogParser)



# -*- coding: utf-8 -*-
"""Parser for Google Chrome autofill entries.

The Chrome autofill information is stored in SQLite database files named
Web Data.
"""

from __future__ import unicode_literals

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.parsers import sqlite
from plaso.parsers.sqlite_plugins import interface


class ChromeAutofillEventData(events.EventData):
  """Chrome Autofill event data.

  Attributes:
    field_name (str): name of form field.
    value (str): value populated in form field.
    usage_count (int): count of times value has been used in field_name.
  """

  DATA_TYPE = 'chrome:autofill:entry'

  def __init__(self):
    """Initializes event data."""
    super(ChromeAutofillEventData, self).__init__(
        data_type=self.DATA_TYPE)
    self.field_name = None
    self.usage_count = None
    self.value = None


class ChromeAutofillPlugin(interface.SQLitePlugin):
  """Plugin to parse Chrome Autofill entries from Web Data database files."""

  NAME = 'chrome_autofill'
  DESCRIPTION = 'Parser for Chrome autofill SQLite database files.'

  # Define the needed queries.
  QUERIES = [
      (('SELECT autofill.date_created, autofill.date_last_used, autofill.name, '
        'autofill.value, autofill.count FROM autofill ORDER BY date_created'),
       'ParseAutofillRow')]

  REQUIRED_TABLES = frozenset(['autofill'])

  SCHEMAS = [{
      'autofill': (
          'CREATE TABLE autofill (name VARCHAR, value VARCHAR, '
          'value_lower VARCHAR, date_created INTEGER DEFAULT 0, '
          'date_last_used INTEGER DEFAULT 0, count INTEGER DEFAULT 1, '
          'PRIMARY KEY (name, value));)')}]

  def ParseAutofillRow(
      self, parser_mediator, query, row, **unused_kwargs):
    """Parses an autofill entry row.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      query (str): query that created the row.
      row (sqlite3.Row): row.
    """
    query_hash = hash(query)

    event_data = ChromeAutofillEventData()
    event_data.field_name = self._GetRowValue(query_hash, row, 'name')
    event_data.value = self._GetRowValue(query_hash, row, 'value')
    event_data.usage_count = self._GetRowValue(query_hash, row, 'count')
    event_data.query = query

    # Create one event for the first time an autofill entry was used
    timestamp = self._GetRowValue(query_hash, row, 'date_created')
    date_time = dfdatetime_posix_time.PosixTime(timestamp=timestamp)
    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_CREATION)
    parser_mediator.ProduceEventWithEventData(event, event_data)

    # If the autofill value has been used more than once, create another
    # event for the most recent time it was used.
    if event_data.usage_count > 1:
      timestamp = self._GetRowValue(query_hash, row, 'date_last_used')
      date_time = dfdatetime_posix_time.PosixTime(timestamp=timestamp)
      event = time_events.DateTimeValuesEvent(
          date_time, definitions.TIME_DESCRIPTION_LAST_USED)
      parser_mediator.ProduceEventWithEventData(event, event_data)


sqlite.SQLiteParser.RegisterPlugin(ChromeAutofillPlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the compound ZIP archive parser."""

from __future__ import unicode_literals

import unittest

from plaso.parsers import czip
# Register all plugins.
from plaso.parsers import czip_plugins  # pylint: disable=unused-import

from tests.parsers import test_lib


class CompoundZIPTest(test_lib.ParserTestCase):
  """Tests for the compound ZIP archive parser."""

  # pylint: disable=protected-access

  def testEnablePlugins(self):
    """Tests the EnablePlugins function."""
    parser = czip.CompoundZIPParser()
    parser.EnablePlugins(['oxml'])

    self.assertIsNotNone(parser)
    self.assertIsNone(parser._default_plugin)
    self.assertNotEqual(parser._plugins, [])
    self.assertEqual(len(parser._plugins), 1)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for parser and parser plugin presets."""

from __future__ import unicode_literals

import unittest

from plaso.containers import artifacts
from plaso.parsers import presets

from tests import test_lib as shared_test_lib


class ParserPresetTest(shared_test_lib.BaseTestCase):
  """Tests for the parser and parser plugin preset."""

  def testInitialize(self):
    """Tests the __init__ function."""
    test_definition = presets.ParserPreset('test', ['parser1', 'parser2'])
    self.assertIsNotNone(test_definition)


class ParserPresetsManagerTest(shared_test_lib.BaseTestCase):
  """Tests for the parser and parser plugin presets manager."""

  _LINUX_PARSERS = [
      'bash_history',
      'bencode',
      'czip/oxml',
      'dockerjson',
      'dpkg',
      'filestat',
      'gdrive_synclog',
      'olecf',
      'pls_recall',
      'popularity_contest',
      'selinux',
      'sqlite/google_drive',
      'sqlite/skype',
      'sqlite/zeitgeist',
      'syslog',
      'systemd_journal',
      'utmp',
      'vsftpd',
      'webhist',
      'xchatlog',
      'xchatscrollback',
      'zsh_extended_history']

  _MACOS_PARSERS = [
      'asl_log',
      'bash_history',
      'bencode',
      'bsm_log',
      'cups_ipp',
      'czip/oxml',
      'filestat',
      'fseventsd',
      'gdrive_synclog',
      'mac_appfirewall_log',
      'mac_keychain',
      'mac_securityd',
      'macwifi',
      'olecf',
      'plist',
      'sqlite/appusage',
      'sqlite/google_drive',
      'sqlite/imessage',
      'sqlite/ls_quarantine',
      'sqlite/mac_document_versions',
      'sqlite/mackeeper_cache',
      'sqlite/skype',
      'syslog',
      'utmpx',
      'webhist',
      'zsh_extended_history']

  # TODO add tests for _ReadPresetDefinitionValues
  # TODO add tests for _ReadPresetsFromFileObject

  def testGetNames(self):
    """Tests the GetNames function."""
    test_file_path = self._GetTestFilePath(['presets.yaml'])
    self._SkipIfPathNotExists(test_file_path)

    test_manager = presets.ParserPresetsManager()
    test_manager.ReadFromFile(test_file_path)

    test_names = list(test_manager.GetNames())
    self.assertEqual(len(test_names), 7)

    expected_names = sorted([
        'android', 'linux', 'macos', 'webhist', 'win7', 'win_gen', 'winxp'])
    self.assertEqual(test_names, expected_names)

  def testGetParsersByPreset(self):
    """Tests the GetParsersByPreset function."""
    test_file_path = self._GetTestFilePath(['presets.yaml'])
    self._SkipIfPathNotExists(test_file_path)

    test_manager = presets.ParserPresetsManager()
    test_manager.ReadFromFile(test_file_path)

    parser_names = test_manager.GetParsersByPreset('linux')
    self.assertEqual(parser_names, self._LINUX_PARSERS)

    with self.assertRaises(KeyError):
      test_manager.GetParsersByPreset('bogus')

  def testGetPresetByName(self):
    """Tests the GetPresetByName function."""
    test_file_path = self._GetTestFilePath(['presets.yaml'])
    self._SkipIfPathNotExists(test_file_path)

    test_manager = presets.ParserPresetsManager()
    test_manager.ReadFromFile(test_file_path)

    test_preset = test_manager.GetPresetByName('linux')
    self.assertIsNotNone(test_preset)
    self.assertEqual(test_preset.name, 'linux')

    self.assertEqual(test_preset.parsers, self._LINUX_PARSERS)

    test_preset = test_manager.GetPresetByName('bogus')
    self.assertIsNone(test_preset)

  def testGetPresetsByOperatingSystem(self):
    """Tests the GetPresetsByOperatingSystem function."""
    test_file_path = self._GetTestFilePath(['presets.yaml'])
    self._SkipIfPathNotExists(test_file_path)

    test_manager = presets.ParserPresetsManager()
    test_manager.ReadFromFile(test_file_path)

    operating_system = artifacts.OperatingSystemArtifact(family='MacOS')

    test_presets = test_manager.GetPresetsByOperatingSystem(operating_system)
    self.assertEqual(len(test_presets), 1)
    self.assertEqual(test_presets[0].name, 'macos')

    self.assertEqual(test_presets[0].parsers, self._MACOS_PARSERS)

    operating_system = artifacts.OperatingSystemArtifact(family='bogus')

    test_presets = test_manager.GetPresetsByOperatingSystem(operating_system)
    self.assertEqual(len(test_presets), 0)

  def testGetPresetsInformation(self):
    """Tests the GetPresetsInformation function."""
    test_file_path = self._GetTestFilePath(['presets.yaml'])
    self._SkipIfPathNotExists(test_file_path)

    test_manager = presets.ParserPresetsManager()
    test_manager.ReadFromFile(test_file_path)

    parser_presets_information = test_manager.GetPresetsInformation()
    self.assertGreaterEqual(len(parser_presets_information), 1)

    available_parser_names = [name for name, _ in parser_presets_information]
    self.assertIn('linux', available_parser_names)

  # TODO add tests for ReadFromFile


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the PE file parser."""

from __future__ import unicode_literals

import unittest

from plaso.formatters import pe as _  # pylint: disable=unused-import
from plaso.parsers import pe

from tests.parsers import test_lib


class PECOFFTest(test_lib.ParserTestCase):
  """Tests for the PE file parser."""

  def testParseFileObjectOnExecutable(self):
    """Tests the ParseFileObject on a PE executable (EXE) file."""
    parser = pe.PEParser()
    storage_writer = self._ParseFile(['test_pe.exe'], parser)

    self.assertEqual(storage_writer.number_of_warnings, 0)
    self.assertEqual(storage_writer.number_of_events, 3)

    events = list(storage_writer.GetEvents())

    event = events[0]

    self.CheckTimestamp(event.timestamp, '2015-04-21 14:53:56.000000')

    event_data = self._GetEventDataOfEvent(storage_writer, event)
    self.assertEqual(event_data.data_type, 'pe:compilation:compilation_time')
    self.assertEqual(event_data.pe_type, 'Executable (EXE)')

    event = events[1]

    self.CheckTimestamp(event.timestamp, '2015-04-21 14:53:55.000000')

    event_data = self._GetEventDataOfEvent(storage_writer, event)
    self.assertEqual(event_data.data_type, 'pe:import:import_time')

    event = events[2]

    self.CheckTimestamp(event.timestamp, '2015-04-21 14:53:54.000000')

    event_data = self._GetEventDataOfEvent(storage_writer, event)
    self.assertEqual(event_data.data_type, 'pe:delay_import:import_time')

  def testParseFileObjectOnDriver(self):
    """Tests the ParseFileObject on a PE driver (SYS) file."""
    parser = pe.PEParser()
    storage_writer = self._ParseFile(['test_driver.sys'], parser)

    self.assertEqual(storage_writer.number_of_warnings, 0)
    self.assertEqual(storage_writer.number_of_events, 1)

    events = list(storage_writer.GetEvents())

    event = events[0]

    self.CheckTimestamp(event.timestamp, '2015-04-21 14:53:54.000000')

    event_data = self._GetEventDataOfEvent(storage_writer, event)
    self.assertEqual(event_data.pe_type, 'Driver (SYS)')


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The Bash history event formatter."""

from __future__ import unicode_literals

from plaso.formatters import interface
from plaso.formatters import manager


class BashHistoryEventFormatter(interface.EventFormatter):
  """Formatter for Bash history events."""

  DATA_TYPE = 'bash:history:command'

  FORMAT_STRING = 'Command executed: {command}'
  FORMAT_STRING_SHORT = '{command}'

  SOURCE_SHORT = 'LOG'
  SOURCE_LONG = 'Bash History'


manager.FormattersManager.RegisterFormatter(BashHistoryEventFormatter)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the XLSX output module."""

from __future__ import unicode_literals

import os
import unittest
import zipfile

from defusedxml import ElementTree

from plaso.formatters import interface as formatters_interface
from plaso.formatters import manager as formatters_manager
from plaso.lib import definitions
from plaso.lib import timelib
from plaso.output import xlsx

from tests import test_lib as shared_test_lib
from tests.containers import test_lib as containers_test_lib
from tests.output import test_lib


class TestEventFormatter(formatters_interface.EventFormatter):
  """Event object formatter used for testing."""

  DATA_TYPE = 'test:xlsx'
  FORMAT_STRING = '{text}'

  SOURCE_SHORT = 'LOG'
  SOURCE_LONG = 'Syslog'


class XLSXOutputModuleTest(test_lib.OutputModuleTestCase):
  """Test the XLSX output module."""

  _SHARED_STRINGS = 'xl/sharedStrings.xml'
  _SHEET1 = 'xl/worksheets/sheet1.xml'

  _COLUMN_TAG = '}c'
  _ROW_TAG = '}row'
  _SHARED_STRING_TAG = '}t'
  _SHARED_STRING_TYPE = 's'
  _TYPE_ATTRIBUTE = 't'
  _VALUE_STRING_TAG = '}v'

  _TEST_EVENTS = [
      {'data_type': 'test:xlsx',
       'hostname': 'ubuntu',
       'filename': 'log/syslog.1',
       'text': (
           'Reporter <CRON> PID: 8442 (pam_unix(cron:session): session\n '
           'closed for user root) Invalid character -> \ud801'),
       'timestamp': timelib.Timestamp.CopyFromString('2012-06-27 18:17:01'),
       'timestamp_desc': definitions.TIME_DESCRIPTION_CHANGE}]

  def _GetSheetRows(self, filename):
    """Parses the contents of the first sheet of an XLSX document.

    Args:
      filename (str): The file path of the XLSX document to parse.

    Returns:
      list[list[str]]: A list of lists representing the rows of the first sheet.

    Raises:
      ValueError: if the sheet cannot be found, or a string cannot be read.
    """
    zip_file = zipfile.ZipFile(filename)

    # Fail if we can't find the expected first sheet.
    if self._SHEET1 not in zip_file.namelist():
      raise ValueError(
          'Unable to locate expected sheet: {0:s}'.format(self._SHEET1))

    # Generate a reference table of shared strings if available.
    strings = []
    if self._SHARED_STRINGS in zip_file.namelist():
      zip_file_object = zip_file.open(self._SHARED_STRINGS)
      for _, element in ElementTree.iterparse(zip_file_object):
        if element.tag.endswith(self._SHARED_STRING_TAG):
          strings.append(element.text)

    row = []
    rows = []
    value = ''
    zip_file_object = zip_file.open(self._SHEET1)
    for _, element in ElementTree.iterparse(zip_file_object):
      if (element.tag.endswith(self._VALUE_STRING_TAG) or
          element.tag.endswith(self._SHARED_STRING_TAG)):
        value = element.text

      if element.tag.endswith(self._COLUMN_TAG):
        # Grab value from shared string reference table if type shared string.
        if (strings and element.attrib.get(self._TYPE_ATTRIBUTE) ==
            self._SHARED_STRING_TYPE):
          try:
            value = strings[int(value)]
          except (IndexError, ValueError):
            raise ValueError(
                'Unable to successfully dereference shared string.')

        row.append(value)

      # If we see the end tag of the row, record row in rows and reset.
      if element.tag.endswith(self._ROW_TAG):
        rows.append(row)
        row = []

    return rows

  def testWriteEventBody(self):
    """Tests the WriteHeader function."""
    formatters_manager.FormattersManager.RegisterFormatter(TestEventFormatter)

    expected_header = [
        'datetime', 'timestamp_desc', 'source', 'source_long',
        'message', 'parser', 'display_name', 'tag']
    expected_event_body = [
        '41087.76181712963', 'Metadata Modification Time', 'LOG', 'Syslog',
        'Reporter <CRON> PID: 8442 (pam_unix(cron:session): session '
        'closed for user root) Invalid character -> \ufffd',
        '-', '-', '-']

    with shared_test_lib.TempDirectory() as temp_directory:
      output_mediator = self._CreateOutputMediator()
      output_module = xlsx.XLSXOutputModule(output_mediator)

      xslx_file = os.path.join(temp_directory, 'xlsx.out')
      output_module.SetFilename(xslx_file)

      output_module.Open()
      output_module.WriteHeader()

      event, event_data = containers_test_lib.CreateEventFromValues(
          self._TEST_EVENTS[0])
      output_module.WriteEvent(event, event_data, None)

      output_module.WriteFooter()
      output_module.Close()

      try:
        rows = self._GetSheetRows(xslx_file)
      except ValueError as exception:
        self.fail(exception)

      self.assertEqual(expected_header, rows[0])
      self.assertEqual(len(expected_event_body), len(rows[1]))
      self.assertEqual(expected_event_body, rows[1])

  def testWriteHeader(self):
    """Tests the WriteHeader function."""
    expected_header = [
        'datetime', 'timestamp_desc', 'source', 'source_long',
        'message', 'parser', 'display_name', 'tag']

    with shared_test_lib.TempDirectory() as temp_directory:
      output_mediator = self._CreateOutputMediator()
      output_module = xlsx.XLSXOutputModule(output_mediator)

      xlsx_file = os.path.join(temp_directory, 'xlsx.out')
      output_module.SetFilename(xlsx_file)

      output_module.Open()
      output_module.WriteHeader()
      output_module.WriteFooter()
      output_module.Close()

      try:
        rows = self._GetSheetRows(xlsx_file)
      except ValueError as exception:
        self.fail(exception)

      self.assertEqual(expected_header, rows[0])


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Helper for parser and plugin filter expressions."""

from __future__ import unicode_literals


class ParserFilterExpressionHelper(object):
  """Helper for parser and plugin filter expressions.

  A parser filter expression is a comma separated value string that denotes
  which parsers and plugins should be used. Each element can contain either:

  * The name of a preset (case sensitive), which is a predefined list of parsers
    and/or plugins (see data/presets.yaml for the default presets).
  * The name of a parser (case insensitive), for example 'msiecf'.
  * The name of a plugin, prefixed with the parser name and a '/', for example
    'sqlite/chrome_history'.

  If the element begins with an exclamation mark ('!') the item will be
  excluded from the set of enabled parsers and plugins, otherwise the element
  will be included.
  """

  def _GetParserAndPluginsList(self, parsers_and_plugins):
    """Flattens the parsers and plugins dictionary into a list.

    Args:
      parsers_and_plugins (dict[str, set[str]]): parsers and plugins.

    Returns:
      list[str]: alphabetically sorted list of the parsers and plugins.
    """
    parser_filters = []
    for parser_name, plugins in sorted(parsers_and_plugins.items()):
      for plugin_name in sorted(plugins):
        if plugin_name == '*':
          parser_filters.append(parser_name)
        else:
          parser_filters.append('/'.join([parser_name, plugin_name]))

    return parser_filters

  def _JoinExpression(self, excludes, includes):
    """Creates an expression string of the excluded and included parsers.

    Args:
      excludes (dict[str, set[str]]): excluded parsers and plugins.
      includes (dict[str, set[str]]): included parsers and plugins.

    Returns:
      str: a parser filter expression.

        A parser filter expression is a comma separated value string that
        denotes which parsers should be used. Each element can contain either:

        * The name of a parser (case insensitive), for example 'msiecf'.
        * The name of a plugin, prefixed with the parser name and a '/', for
          example 'sqlite/chrome_history'.

        If the element begins with an exclamation mark ('!') the item will be
        excluded from the set of enabled parsers and plugins, otherwise the
        element will be included.

    Raises:
      RuntimeError: if a specific plugin is excluded but no corresponding parser
          is included or if exclude and include parser filters overlap.
    """
    excluded_parsers_and_plugins = self._GetParserAndPluginsList(excludes)
    included_parsers_and_plugins = self._GetParserAndPluginsList(includes)

    # Note that below set comprehension is used to determine the set of
    # excluded and included parsers.
    excluded_parsers = {
        name for name, plugins in excludes.items() if plugins != set(['*'])}
    included_parsers = {
        name for name, plugins in includes.items() if '*' in plugins}
    missing_parser_filters = excluded_parsers.difference(included_parsers)
    if missing_parser_filters:
      missing_parser_filters = self._GetParserAndPluginsList({
          name:  excludes[name] for name in missing_parser_filters})
      raise RuntimeError((
          'Parser filters: {0:s} defined to be excluded but no corresponding '
          'include expression').format(','.join(missing_parser_filters)))

    overlapping_parser_filters = set(included_parsers_and_plugins).intersection(
        set(excluded_parsers_and_plugins))
    if overlapping_parser_filters:
      raise RuntimeError((
          'Parser filters: {0:s} defined to be both included and '
          'excluded').format(','.join(overlapping_parser_filters)))

    parser_filters = [
        '!{0:s}'.format(parser_filter)
        for parser_filter in excluded_parsers_and_plugins]
    parser_filters.extend(included_parsers_and_plugins)
    return ','.join(parser_filters)

  def _ExpandPreset(self, presets_manager, preset_name, parsers_and_plugins):
    """Expands a preset in the parsers and plugins.

    This functions replaces the preset in parsers_and_plugins with the parser
    and plugin or preset names defined by the preset.

    Args:
      presets_manager (ParserPresetsManager): a parser preset manager, that
          is used to resolve which parsers and/or plugins are defined by
          presets.
      preset_name (str): name of the preset to expand.
      parsers_and_plugins (dict[str, set[str]]): parsers, plugins and presets.

    Raises:
      RuntimeError: if the plugins list of a preset in parsers_and_plugins,
          contains plugin definitions other than '*'.
    """
    if preset_name not in parsers_and_plugins:
      return

    plugins = parsers_and_plugins.get(preset_name, None)
    if plugins != set(['*']):
      raise RuntimeError((
          '{0:s} cannot be used as a preset name and plugin name at '
          'same time.').format(preset_name))

    del parsers_and_plugins[preset_name]

    for parser_filter in presets_manager.GetParsersByPreset(preset_name):
      parser, _, plugin = parser_filter.partition('/')
      if not plugin:
        plugin = '*'

      parsers_and_plugins.setdefault(parser, set())
      parsers_and_plugins[parser].add(plugin)

  def _ExpandPresets(self, presets_manager, preset_names, parsers_and_plugins):
    """Expands the presets in the parsers and plugins.

    This functions replaces the presets in parsers_and_plugins with the parser
    and plugin or preset names defined by the presets.

    Args:
      presets_manager (ParserPresetsManager): a parser preset manager, that
          is used to resolve which parsers and/or plugins are defined by
          presets.
      preset_names (set[str]): names of the presets defined by the presets
          manager.
      parsers_and_plugins (dict[str, set[str]]): parsers, plugins and presets.
    """
    for preset_name in preset_names:
      self._ExpandPreset(presets_manager, preset_name, parsers_and_plugins)

  def ExpandPresets(self, presets_manager, expression):
    """Expands all presets in a parser filter expression.

    Args:
      presets_manager (ParserPresetsManager): a parser preset manager, that
          is used to resolve which parsers and/or plugins are defined by
          presets.
      expression (str): parser filter expression, where None represents all
          parsers and plugins.

          A parser filter expression is a comma separated value string that
          denotes which parsers and plugins should be used. Each element can be
          either:

          * The name of a preset (case sensitive), which is a predefined list of
            parsers and/or plugins (see data/presets.yaml for the default
            presets).
          * The name of a parser (case insensitive), for example 'msiecf'.
          * The name of a plugin, prefixed with the parser name and a '/', for
            example 'sqlite/chrome_history'.

          If the element begins with an exclamation mark ('!') the item will
          be excluded from the set of enabled parsers and plugins, otherwise
          the element will be included.

    Returns:
      str: a parser filter expression where presets have been expanded or None
          to represent all parsers and plugins.
    """
    preset_names = set(presets_manager.GetNames())

    excludes, includes = self.SplitExpression(expression)

    while set(excludes.keys()).intersection(preset_names):
      self._ExpandPresets(presets_manager, preset_names, excludes)

    while set(includes.keys()).intersection(preset_names):
      self._ExpandPresets(presets_manager, preset_names, includes)

    return self._JoinExpression(excludes, includes)

  def SplitExpression(self, expression):
    """Determines the excluded and included elements in an expression string.

    This method will not expand presets, and preset names are treated like
    parser names.

    Args:
      expression (str): parser filter expression, where None represents all
          parsers and plugins.

          A parser filter expression is a comma separated value string that
          denotes which parsers and plugins should be used. Each element can be
          either:

          * The name of a preset (case sensitive), which is a predefined list of
            parsers and/or plugins (see data/presets.yaml for the default
            presets).
          * The name of a parser (case insensitive), for example 'msiecf'.
          * The name of a plugin, prefixed with the parser name and a '/', for
            example 'sqlite/chrome_history'.

          If the element begins with an exclamation mark ('!') the item will
          be excluded from the set of enabled parsers and plugins, otherwise
          the element will be included.

    Returns:
      tuple: contains:

        excludes (dict[str, set[str]]): excluded presets, plugins and presets.
            Dictionary keys are preset and/or parser names, and values are
            sets containing plugin names to enable for a parser or an asterisk
            character ('*') to represet all plugins, or that no specific
            plugins were specified.
        includes (dict[str, set[str]]): included presets, parsers and plugins.
            Dictionary keys are preset and/or parser names, and values are
            sets containing plugin names to enable for a parser or an asterisk
            character ('*') to represet all plugins, or that no specific
            plugins were specified.
    """
    if not expression:
      return {}, {}

    excludes = {}
    includes = {}

    for expression_element in expression.split(','):
      expression_element = expression_element.strip()
      if not expression_element:
        continue

      expression_element = expression_element.lower()

      if expression_element.startswith('!'):
        expression_element = expression_element[1:]
        parsers_and_plugins = excludes
      else:
        parsers_and_plugins = includes

      parser, _, plugin = expression_element.partition('/')
      if not plugin:
        plugin = '*'

      parsers_and_plugins.setdefault(parser, set())
      parsers_and_plugins[parser].add(plugin)

    return excludes, includes



# -*- coding: utf-8 -*-
"""This file contains a parser for compound ZIP files."""

from __future__ import unicode_literals

import struct
import zipfile

from plaso.lib import errors
from plaso.parsers import interface
from plaso.parsers import logger
from plaso.parsers import manager


class CompoundZIPParser(interface.FileObjectParser):
  """Shared functionality for parsing compound zip files.

  Compound zip files are zip files used as containers to create another file
  format, as opposed to archives of unrelated files.
  """

  NAME = 'czip'
  DESCRIPTION = 'Parser for compound ZIP files.'

  _plugin_classes = {}

  def ParseFileObject(self, parser_mediator, file_object):
    """Parses a compound ZIP file-like object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_object (dfvfs.FileIO): a file-like object.

    Raises:
      UnableToParseFile: when the file cannot be parsed.
    """
    display_name = parser_mediator.GetDisplayName()

    if not zipfile.is_zipfile(file_object):
      raise errors.UnableToParseFile(
          '[{0:s}] unable to parse file: {1:s} with error: {2:s}'.format(
              self.NAME, display_name, 'Not a Zip file.'))

    try:
      zip_file = zipfile.ZipFile(file_object, 'r', allowZip64=True)
      self._ProcessZipFileWithPlugins(parser_mediator, zip_file)
      zip_file.close()

    # Some non-ZIP files return true for is_zipfile but will fail with a
    # negative seek (IOError) or another error.
    except (zipfile.BadZipfile, struct.error) as exception:
      raise errors.UnableToParseFile(
          '[{0:s}] unable to parse file: {1:s} with error: {2!s}'.format(
              self.NAME, display_name, exception))

  def _ProcessZipFileWithPlugins(self, parser_mediator, zip_file):
    """Processes a zip file using all compound zip files.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      zip_file (zipfile.ZipFile): the zip file. It should not be closed in
          this method, but will be closed in ParseFileObject().
    """
    archive_members = zip_file.namelist()
    for plugin in self._plugins:
      try:
        plugin.UpdateChainAndProcess(
            parser_mediator, zip_file=zip_file, archive_members=archive_members)
      except errors.WrongCompoundZIPPlugin as exception:
        logger.debug('[{0:s}] wrong plugin: {1!s}'.format(
            self.NAME, exception))


manager.ParsersManager.RegisterParser(CompoundZIPParser)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the .customDestinations-ms file parser."""

from __future__ import unicode_literals

import unittest

from plaso.formatters import winlnk  # pylint: disable=unused-import
from plaso.lib import definitions
from plaso.parsers import custom_destinations

from tests.parsers import test_lib


class CustomDestinationsParserTest(test_lib.ParserTestCase):
  """Tests for the .customDestinations-ms file parser."""

  def testParse(self):
    """Tests the Parse function."""
    parser = custom_destinations.CustomDestinationsParser()
    storage_writer = self._ParseFile(
        ['5afe4de1b92fc382.customDestinations-ms'], parser)

    self.assertEqual(storage_writer.number_of_warnings, 0)
    self.assertEqual(storage_writer.number_of_events, 126)

    events = list(storage_writer.GetEvents())

    # A shortcut event.
    # The last accessed timestamp.
    event = events[121]

    self.CheckTimestamp(event.timestamp, '2009-07-13 23:55:56.248104')
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_LAST_ACCESS)

    # The creation timestamp.
    event = events[122]

    self.CheckTimestamp(event.timestamp, '2009-07-13 23:55:56.248104')
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_CREATION)

    # The last modification timestamp.
    event = events[123]

    self.CheckTimestamp(event.timestamp, '2009-07-14 01:39:11.388000')
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_MODIFICATION)

    event_data = self._GetEventDataOfEvent(storage_writer, event)

    expected_message = (
        '[@%systemroot%\\system32\\oobefldr.dll,-1262] '
        'File size: 11776 '
        'File attribute flags: 0x00000020 '
        'Drive type: 3 '
        'Drive serial number: 0x24ba718b '
        'Local path: C:\\Windows\\System32\\GettingStarted.exe '
        'cmd arguments: {DE3895CB-077B-4C38-B6E3-F3DE1E0D84FC} '
        '%systemroot%\\system32\\control.exe /name Microsoft.Display '
        'env location: %SystemRoot%\\system32\\GettingStarted.exe '
        'Icon location: %systemroot%\\system32\\display.dll '
        'Link target: <My Computer> C:\\Windows\\System32\\GettingStarted.exe')

    expected_short_message = (
        '[@%systemroot%\\system32\\oobefldr.dll,-1262] '
        'C:\\Windows\\System32\\GettingStarte...')

    self._TestGetMessageStrings(
        event_data, expected_message, expected_short_message)

    # A shell item event.
    event = events[18]

    self.CheckTimestamp(event.timestamp, '2010-11-10 07:41:04.000000')

    event_data = self._GetEventDataOfEvent(storage_writer, event)

    expected_message = (
        'Name: System32 '
        'Long name: System32 '
        'NTFS file reference: 2331-1 '
        'Shell item path: <My Computer> C:\\Windows\\System32 '
        'Origin: 5afe4de1b92fc382.customDestinations-ms')

    expected_short_message = (
        'Name: System32 '
        'NTFS file reference: 2331-1 '
        'Origin: 5afe4de1b92fc382.customDes...')

    self._TestGetMessageStrings(
        event_data, expected_message, expected_short_message)

    # A distributed link tracking event.
    event = events[12]

    self.CheckTimestamp(event.timestamp, '2010-11-10 19:08:32.656260')

    event_data = self._GetEventDataOfEvent(storage_writer, event)

    expected_message = (
        'e9215b24-ecfd-11df-a81c-000c29031e1e '
        'MAC address: 00:0c:29:03:1e:1e '
        'Origin: 5afe4de1b92fc382.customDestinations-ms')

    expected_short_message = (
        'e9215b24-ecfd-11df-a81c-000c29031e1e '
        'Origin: 5afe4de1b92fc382.customDestinati...')

    self._TestGetMessageStrings(
        event_data, expected_message, expected_short_message)


if __name__ == '__main__':
  unittest.main()
